#!/bin/bash
# This script creates patches for the qt2-port. Threrfore we need a reference directory which is already patched to compile
# with kdelibs 3.5.
# Written by Eva Brucherseifer, changed by Stefan Eilers

set +x

# DEVDIR is the base directory of your development. This dirctory should contain the directory "kdelibs" and "kdenox"
export DEVDIR=$HOME/DevProjects/konqe/

#change this to your directories
#REFDIR:    This is the reference directory for diff. It contains the kdesrc directory, patched for the Qt3 release of konq-embedded 
#CHANGEDIR: The directory where changes are made.
#TARGETDIR: The directory where patches are stored. In normal cases it is stored in the CHANGEDIR
export REFDIR=$DEVDIR/qte3/kdenox/konq-embed/kdesrc/
export CHANGEDIR=$DEVDIR/kdenox/konq-embed/kdesrc/
export TARGETDIR=$CHANGEDIR


echo "Creating Patches..."

( cd $REFDIR/kdecore && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kdecore/ > $TARGETDIR/kdecore/patches/kdecore_qt2.3.patch )
( cd $REFDIR/kdecore/malloc && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kdecore/malloc > $TARGETDIR/kdecore/patches/malloc_qt2.3.patch )
( cd $REFDIR/kdecore/network && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kdecore/network > $TARGETDIR/kdecore/patches/network_qt2.3.patch )


( cd $REFDIR/khtml/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/ > $TARGETDIR/khtml/patches/khtml_qt2.3.patch )
( cd $REFDIR/khtml/css && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/css > $TARGETDIR/khtml/patches/css_qt2.3.patch )
( cd $REFDIR/khtml/dom && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/dom > $TARGETDIR/khtml/patches/dom_qt2.3.patch )
( cd $REFDIR/khtml/ecma && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/ecma > $TARGETDIR/khtml/patches/ecma_qt2.3.patch )
( cd $REFDIR/khtml/html && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/html > $TARGETDIR/khtml/patches/html_qt2.3.patch )
( cd $REFDIR/khtml/misc && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/misc > $TARGETDIR/khtml/patches/misc_qt2.3.patch )
( cd $REFDIR/khtml/rendering && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/rendering > $TARGETDIR/khtml/patches/rendering_qt2.3.patch )
( cd $REFDIR/khtml/xml && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/khtml/xml > $TARGETDIR/khtml/patches/xml_qt2.3.patch )


( cd $REFDIR/kio/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kio/ > $TARGETDIR/kio/patches/kio_qt2.3.patch )
( cd $REFDIR/kio/http && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kio/http > $TARGETDIR/kio/patches/kio_http_qt2.3.patch )
( cd $REFDIR/kio/http/kcookiejar && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kio/http/kcookiejar > $TARGETDIR/kio/patches/kcookiejar_qt2.3.patch )
( cd $REFDIR/kio/kfile && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kio/kfile > $TARGETDIR/kio/patches/kfile_qt2.3.patch )

( cd $REFDIR/kjs/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kjs/ > $TARGETDIR/kjs/patches/kjs_qt2.3.patch )

( cd $REFDIR/kparts/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kparts/ > $TARGETDIR/kparts/patches/kparts_qt2.3.patch )

( cd $REFDIR/kssl/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kssl/ > $TARGETDIR/kssl/patches/kssl_qt2.3.patch )

( cd $REFDIR/kutils/ && diff -b -u -p -X $DEVDIR/kdenox/devscripts/patch_excludepatterns.txt . $CHANGEDIR/kutils/ > $TARGETDIR/kutils/patches/kutils_qt2.3.patch )

#( cd $TARGETDIR && find . -name "*.patch" -print > replacelist.txt \; )


echo "Cleaning up patches..."
PWD=`pwd`
cd $TARGETDIR
for fl in $(find . -name "*.patch" -print) ; do
  mv $fl $fl.old
  sed -e '/^Gemeinsame Unterverzeichnisse/d' -e '/^Nur in/d' -e '/^Only in/d' -e '/^Common subdirectories/d' -e '/^diff -b -u/d' -e "s#$REFDIR/##g" -e "s#$CHANGEDIR/##g" -e 's#2006-.*$##g'  $fl.old > $fl
  rm $fl.old
done

cd $PWD
