/* This file is part of the KDE libraries
   Copyright (C) 2000 Matej Koss <koss@miesto.sk>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "jobclasses.h"
#include "progressbase.h"

//namespace KIO {

ProgressBase::ProgressBase( QWidget *parent, const char * name, bool modal, WFlags f  )
  : QDialog( parent,name, modal, f )
{
  m_pJob = 0;

  // delete dialog after the job is finished / canceled
  m_bOnlyClean = false;

  // stop job on close
  m_bStopOnClose = true;
}

ProgressBase::~ProgressBase()
{
  //slotStop();
}

void ProgressBase::setJob( KIO::Job *job )
{
  // first connect all slots
  connect( job, SIGNAL( percent( KIO::Job*, unsigned long ) ),
	   SLOT( slotPercent( KIO::Job*, unsigned long ) ) );

  connect( job, SIGNAL( result( KIO::Job* ) ),
	   SLOT( slotFinished( KIO::Job* ) ) );

  connect( job, SIGNAL( canceled( KIO::Job* ) ),
	   SLOT( slotFinished( KIO::Job* ) ) );

  // then assign job
  m_pJob = job;
}


void ProgressBase::setJob( KIO::IODeviceJob *job )
{
  // first connect all slots
  connect( job, SIGNAL( totalSize( KIO::Job*, KIO::filesize_t ) ), 
	   SLOT( slotTotalSize( KIO::Job*, KIO::filesize_t ) ) );

  connect( job, SIGNAL( processedSize( KIO::Job*, KIO::filesize_t ) ),
	   SLOT( slotProcessedSize( KIO::Job*, KIO::filesize_t ) ) );

  connect( job, SIGNAL( speed( KIO::Job*, unsigned long ) ),
	   SLOT( slotSpeed( KIO::Job*, unsigned long ) ) );
  connect( job, SIGNAL( percent( KIO::Job*, unsigned long ) ),
	   SLOT( slotPercent( KIO::Job*, unsigned long ) ) );

  connect( job, SIGNAL( copying( KIO::Job*, const KURL& , const KURL& ) ),
	   SLOT( slotCopying( KIO::Job*, const KURL&, const KURL& ) ) );

  connect( job, SIGNAL( result( KIO::Job* ) ),
	   SLOT( slotFinished( KIO::Job* ) ) );

  connect( job, SIGNAL( canceled( KIO::Job* ) ),
	   SLOT( slotFinished( KIO::Job* ) ) );

  // then assign job
  m_pJob = job;
}


void ProgressBase::closeEvent( QCloseEvent* ) {
  // kill job when desired
  if ( m_bStopOnClose ) {
    slotStop();
  } else {
    // clean or delete dialog
    if ( m_bOnlyClean ) {
      slotClean();
    } else {
      delete this;
    }
  }
}

void ProgressBase::finished() {
  kdDebug() << "ProgressBase::finished()" << endl;
  // clean or delete dialog
  if ( m_bOnlyClean ) {
    slotClean();
  } else {
        accept();
  }
}

void ProgressBase::slotFinished( KIO::Job* ) {
  finished();
}


void ProgressBase::slotStop() {
  if ( m_pJob ) {
    m_pJob->kill(); // this will call slotFinished
    m_pJob = 0L;
  } else {
    slotFinished( 0 ); // here we call it ourselves
  }

  emit stopped();
  slotClean();
}


void ProgressBase::slotClean() {
  accept();
}

//}

#include "progressbase.moc"

