/* This file is part of the KDE libraries
   Copyright (C) 2000 Matej Koss <koss@miesto.sk>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#ifndef __progressdlg_h__
#define __progressdlg_h__

#include <qlabel.h>

#include <kio/global.h>

#include <qprogressbar.h>

#include "progressbase.h"

class KLineEdit;

//namespace KIO {

/*
 * A default implementation of the progress dialog ProgressBase.
 * ProgressBase
 */
class ProgressDlg : public ProgressBase {

  Q_OBJECT

public:
  ProgressDlg( QWidget* parent, const char* name = 0, bool modal = FALSE, WFlags f = 0  );
  ~ProgressDlg();

  /// Shared with uiserver.cpp
  static QString makePercentString( unsigned long percent,
                                    KIO::filesize_t totalSize );

public slots:
  virtual void slotTotalSize( KIO::Job*, KIO::filesize_t size );
  virtual void slotProcessedSize( KIO::Job*, KIO::filesize_t bytes );

  virtual void slotSpeed( KIO::Job*, unsigned long speed );
  virtual void slotPercent( KIO::Job*, unsigned long percent );
  virtual void slotInfoMessage( KIO::Job *job, const QString & msg );

  virtual void slotCopying( KIO::Job*, const KURL& src, const KURL& dest );
  void slotTransferring( KIO::Job*, const KURL& url );
  virtual void slotCanResume( KIO::Job*, KIO::filesize_t );

  void slotClean();
  void slotDone();

protected:
  void init();
  void showTotals();

  KLineEdit* sourceEdit;
//  QLabel* progressLabel;
  QLabel* speedLabel;
//  QLabel* sizeLabel;
//  QLabel* resumeLabel;

  QProgressBar* m_pProgressBar;

  KIO::filesize_t m_iTotalSize;
  KIO::filesize_t m_iProcessedSize;

  enum ModeType { Copy, Done };
  ModeType mode;

private:
  class ProgressDlgPrivate;
  ProgressDlgPrivate* d;
};

//} /* namespace */

#endif // __progressdlg_h__

