/*  This file is part of the KDE project
    Copyright (C) 2006 basysKom GmbH <info@basyskom.de>
    Copyright (C) 2006 ROAD GmbH <info@road-gmbh.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/
// Compatibilty header file for QT2-Port

#ifndef __QTCOMPAT_H_
#define __QTCOMPAT_H_

#include <qglobal.h>
#if defined QT_VERSION && (QT_VERSION < 0x030000)


typedef long			Q_LONG;		// word up to 64 bit signed
typedef unsigned long		Q_ULONG;	// word up to 64 bit unsigned
typedef long long			Q_LLONG;	// signed long long
typedef unsigned long long		Q_ULLONG;	// unsigned long long



#if defined(Q_OS_MACX) && !defined(QT_LARGEFILE_SUPPORT)
#  define QT_LARGEFILE_SUPPORT 64
#endif
#if defined(QT_LARGEFILE_SUPPORT)

typedef Q_ULLONG QtOffset;
#else

typedef Q_ULONG QtOffset;
#endif

// Taken from qiodevice.h from QT3 but changed to uint
typedef uint Offset;

#if !defined(Q_ASSERT)
#  if defined(QT_CHECK_STATE)
#    if defined(QT_FATAL_ASSERT)
#      define Q_ASSERT(x)  ((x) ? (void)0 : qFatal("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
#    else
#      define Q_ASSERT(x)  ((x) ? (void)0 : qWarning("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
#    endif
#  else
#    define Q_ASSERT(x)
#  endif
#endif

// KHTMLView expects QContextEvent but does not evaluate it. I hope this fix will not break anything.
#define QContextMenuEvent QEvent

#include <qstring.h>
inline bool compat_endsWith( const QString& str, const QString& pattern, bool cs = true )
{
    if ( str.isNull() )
        return pattern.isNull();
    int pos = str.length() - pattern.length();
    if ( pos < 0 )
        return FALSE;
    if ( cs )
    {
        return memcmp((char*) &(str.unicode()[pos]), (char*)pattern.unicode(), pattern.length()*sizeof(QChar)) == 0;
    }
    else
    {
        for ( int i = 0; i < (int) pattern.length(); i++ )
        {
            if ( str.lower().unicode()[pos + i]  != pattern.lower().unicode()[i] )
                return FALSE;
        }
    }
    return TRUE;
}

/*!
    Auxiliary function. Converts the plain text string \a plain to a
    rich text formatted string with any HTML meta-characters escaped.
    from QStyleSheet
*/
inline QString compat_HTMLescape( const QString& plain)
{
    QString rich;
    for ( int i = 0; i < int(plain.length()); ++i )
    {
        if ( plain[i] == '<' )
            rich +="&lt;";
        else if ( plain[i] == '>' )
            rich +="&gt;";
        else if ( plain[i] == '&' )
            rich +="&amp;";
        else
            rich += plain[i];
    }
    return rich;
}

/*
inline QTextStream &operator<<( QTextStream& s, const char* str)
{
	s << str;
}
*/

#include <qmetaobject.h>

#define Q_INLINE_TEMPLATES

/*!
    \internal
 
    Returns TRUE if \a object inherits \a superClass within
    the meta object inheritance chain; otherwise returns FALSE.
 
    \sa inherits()
*/
inline void *qt_inheritedBy( QMetaObject *superClass, const QObject *object )
{
    if (!object)
        return 0;
    register QMetaObject *mo = object->metaObject();
    while (mo)
    {
        if (mo == superClass)
            return (void*)object;
        mo = mo->superClass();
    }
    return 0;
}

template <typename T>
T qt_cast(const QObject *object)
{
    return (T)qt_inheritedBy( ((T)0)->staticMetaObject(), object );
}


//
// Use to avoid "unused parameter" warnings
//
#ifndef Q_UNUSED
#define Q_UNUSED(x) (void)x;
#endif


#include <qdatetime.h>
#include <stdio.h>
#include <time.h>



inline QString compat_toLocalString( QDateTime& dt)
{
    if ( !dt.isValid() )
        return QString::null;
//    int y, m, d;
    //dt.julianToGregorian( jd, y, m, d );
            tm tt;
            memset( &tt, 0, sizeof( tm ) );
            char buf[255];
            tt.tm_mday = dt.date().day();
            tt.tm_mon = dt.date().month() - 1;
            tt.tm_year = dt.date().year() - 1900;

            static const char * avoidEgcsWarning = "%x";
            if ( strftime( buf, sizeof(buf), avoidEgcsWarning, &tt ) )
                return QString::fromLocal8Bit( buf );
    return QString::null;
}



#endif // (QT_VERSION < 0x030000)

#endif // __COMPAT_H_
