/****************************************************************************
** $Id: qt/qxml.h   3.3.5   edited Aug 31 12:17 $
**
** Definition of Qt3XmlSimpleReader and related classes.
**
** Created : 000518
**
** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
**
** This file is part of the xml module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition licenses may use this
** file in accordance with the Qt Commercial License Agreement provided
** with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef QXML_COMPAT_H
#define QXML_COMPAT_H

#ifndef QT_H
#include "qtextstream.h"
#include "qfile.h"
#include "qstring.h"
#include "qstringlist.h"
#include "qvaluevector.h"
#endif // QT_H

#if !defined(QT_MODULE_XML) || defined( QT_LICENSE_PROFESSIONAL ) || defined( QT_INTERNAL_XML )
#define QM_EXPORT_XML
#else
#define QM_EXPORT_XML Q_EXPORT
#endif

#ifndef QT_NO_XML

class Qt3XmlNamespaceSupport;
class Qt3XmlAttributes;
class Qt3XmlContentHandler;
class Qt3XmlDefaultHandler;
class Qt3XmlDTDHandler;
class Qt3XmlEntityResolver;
class Qt3XmlErrorHandler;
class Qt3XmlLexicalHandler;
class Qt3XmlDeclHandler;
class Qt3XmlInputSource;
class Qt3XmlLocator;
class Qt3XmlNamespaceSupport;
class Qt3XmlParseException;

class Qt3XmlReader;
class Qt3XmlSimpleReader;

class Qt3XmlSimpleReaderPrivate;
class Qt3XmlNamespaceSupportPrivate;
class Qt3XmlAttributesPrivate;
class Qt3XmlInputSourcePrivate;
class Qt3XmlParseExceptionPrivate;
class Qt3XmlLocatorPrivate;
class Qt3XmlDefaultHandlerPrivate;


//
// SAX Namespace Support
//

class QM_EXPORT_XML Qt3XmlNamespaceSupport
{
public:
    Qt3XmlNamespaceSupport();
    ~Qt3XmlNamespaceSupport();

    void setPrefix( const QString&, const QString& );

    QString prefix( const QString& ) const;
    QString uri( const QString& ) const;
    void splitName( const QString&, QString&, QString& ) const;
    void processName( const QString&, bool, QString&, QString& ) const;
    QStringList prefixes() const;
    QStringList prefixes( const QString& ) const;

    void pushContext();
    void popContext();
    void reset();

private:
    Qt3XmlNamespaceSupportPrivate *d;

    friend class Qt3XmlSimpleReader;
};


//
// SAX Attributes
//

class QM_EXPORT_XML Qt3XmlAttributes
{
public:
    Qt3XmlAttributes() {}
    virtual ~Qt3XmlAttributes() {}

    int index( const QString& qName ) const;
    int index( const QString& uri, const QString& localPart ) const;
    int length() const;
    int count() const;
    QString localName( int index ) const;
    QString qName( int index ) const;
    QString uri( int index ) const;
    QString type( int index ) const;
    QString type( const QString& qName ) const;
    QString type( const QString& uri, const QString& localName ) const;
    QString value( int index ) const;
    QString value( const QString& qName ) const;
    QString value( const QString& uri, const QString& localName ) const;

    void clear();
    void append( const QString &qName, const QString &uri, const QString &localPart, const QString &value );

private:
    QStringList qnameList;
    QStringList uriList;
    QStringList localnameList;
    QStringList valueList;

    Qt3XmlAttributesPrivate *d;
};

//
// SAX Input Source
//

class QM_EXPORT_XML Qt3XmlInputSource
{
public:
    Qt3XmlInputSource();
    Qt3XmlInputSource( QIODevice *dev );
    Qt3XmlInputSource( QFile& file ); // obsolete
    Qt3XmlInputSource( QTextStream& stream ); // obsolete
    virtual ~Qt3XmlInputSource();

    virtual void setData( const QString& dat );
    virtual void setData( const QByteArray& dat );
    virtual void fetchData();
    virtual QString data();
    virtual QChar next();
    virtual void reset();

    static const QChar EndOfData;
    static const QChar EndOfDocument;

protected:
    virtual QString fromRawData( const QByteArray &data, bool beginning = FALSE );

private:
    void init();

    QIODevice *inputDevice;
    QTextStream *inputStream;

    QString str;
    const QChar *unicode;
    int pos;
    int length;
    bool nextReturnedEndOfData;
    QTextDecoder *encMapper;

    Qt3XmlInputSourcePrivate *d;
};

//
// SAX Exception Classes
//

class QM_EXPORT_XML Qt3XmlParseException
{
public:
    Qt3XmlParseException( const QString& name="", int c=-1, int l=-1, const QString& p="", const QString& s="" )
	: msg( name ), column( c ), line( l ), pub( p ), sys( s )
    { }

    int columnNumber() const;
    int lineNumber() const;
    QString publicId() const;
    QString systemId() const;
    QString message() const;

private:
    QString msg;
    int column;
    int line;
    QString pub;
    QString sys;

    Qt3XmlParseExceptionPrivate *d;
};


//
// XML Reader
//

class QM_EXPORT_XML Qt3XmlReader
{
public:
    virtual bool feature( const QString& name, bool *ok = 0 ) const = 0;
    virtual void setFeature( const QString& name, bool value ) = 0;
    virtual bool hasFeature( const QString& name ) const = 0;
    virtual void* property( const QString& name, bool *ok = 0 ) const = 0;
    virtual void setProperty( const QString& name, void* value ) = 0;
    virtual bool hasProperty( const QString& name ) const = 0;
    virtual void setEntityResolver( Qt3XmlEntityResolver* handler ) = 0;
    virtual Qt3XmlEntityResolver* entityResolver() const = 0;
    virtual void setDTDHandler( Qt3XmlDTDHandler* handler ) = 0;
    virtual Qt3XmlDTDHandler* DTDHandler() const = 0;
    virtual void setContentHandler( Qt3XmlContentHandler* handler ) = 0;
    virtual Qt3XmlContentHandler* contentHandler() const = 0;
    virtual void setErrorHandler( Qt3XmlErrorHandler* handler ) = 0;
    virtual Qt3XmlErrorHandler* errorHandler() const = 0;
    virtual void setLexicalHandler( Qt3XmlLexicalHandler* handler ) = 0;
    virtual Qt3XmlLexicalHandler* lexicalHandler() const = 0;
    virtual void setDeclHandler( Qt3XmlDeclHandler* handler ) = 0;
    virtual Qt3XmlDeclHandler* declHandler() const = 0;
    virtual bool parse( const Qt3XmlInputSource& input ) = 0;
    virtual bool parse( const Qt3XmlInputSource* input ) = 0;
};

class QM_EXPORT_XML Qt3XmlSimpleReader : public Qt3XmlReader
{
public:
    Qt3XmlSimpleReader();
    virtual ~Qt3XmlSimpleReader();

    bool feature( const QString& name, bool *ok = 0 ) const;
    void setFeature( const QString& name, bool value );
    bool hasFeature( const QString& name ) const;

    void* property( const QString& name, bool *ok = 0 ) const;
    void setProperty( const QString& name, void* value );
    bool hasProperty( const QString& name ) const;

    void setEntityResolver( Qt3XmlEntityResolver* handler );
    Qt3XmlEntityResolver* entityResolver() const;
    void setDTDHandler( Qt3XmlDTDHandler* handler );
    Qt3XmlDTDHandler* DTDHandler() const;
    void setContentHandler( Qt3XmlContentHandler* handler );
    Qt3XmlContentHandler* contentHandler() const;
    void setErrorHandler( Qt3XmlErrorHandler* handler );
    Qt3XmlErrorHandler* errorHandler() const;
    void setLexicalHandler( Qt3XmlLexicalHandler* handler );
    Qt3XmlLexicalHandler* lexicalHandler() const;
    void setDeclHandler( Qt3XmlDeclHandler* handler );
    Qt3XmlDeclHandler* declHandler() const;

    bool parse( const Qt3XmlInputSource& input );
    bool parse( const Qt3XmlInputSource* input );
    virtual bool parse( const Qt3XmlInputSource* input, bool incremental );
    virtual bool parseContinue();

private:
    // variables
    Qt3XmlContentHandler *contentHnd;
    Qt3XmlErrorHandler   *errorHnd;
    Qt3XmlDTDHandler     *dtdHnd;
    Qt3XmlEntityResolver *entityRes;
    Qt3XmlLexicalHandler *lexicalHnd;
    Qt3XmlDeclHandler    *declHnd;

    Qt3XmlInputSource *inputSource;

    QChar c; // the character at reading position
    int   lineNr; // number of line
    int   columnNr; // position in line

    int     nameArrayPos;
    QChar   nameArray[256]; // only used for names
    QString nameValue; // only used for names
    int     refArrayPos;
    QChar   refArray[256]; // only used for references
    QString refValue; // only used for references
    int     stringArrayPos;
    QChar   stringArray[256]; // used for any other strings that are parsed
    QString stringValue; // used for any other strings that are parsed

    Qt3XmlSimpleReaderPrivate* d;

    const QString &string();
    void stringClear();
    inline void stringAddC() { stringAddC(c); }
    void stringAddC(const QChar&);
    const QString& name();
    void nameClear();
    inline void nameAddC() { nameAddC(c); }
    void nameAddC(const QChar&);
    const QString& ref();
    void refClear();
    inline void refAddC() { refAddC(c); }
    void refAddC(const QChar&);

    // used by parseReference() and parsePEReference()
    enum EntityRecognitionContext { InContent, InAttributeValue, InEntityValue, InDTD };

    // private functions
    bool eat_ws();
    bool next_eat_ws();

    void next();
    bool atEnd();

    void init( const Qt3XmlInputSource* i );
    void initData();

    bool entityExist( const QString& ) const;

    bool parseBeginOrContinue( int state, bool incremental );

    bool parseProlog();
    bool parseElement();
    bool processElementEmptyTag();
    bool processElementETagBegin2();
    bool processElementAttribute();
    bool parseMisc();
    bool parseContent();

    bool parsePI();
    bool parseDoctype();
    bool parseComment();

    bool parseName();
    bool parseNmtoken();
    bool parseAttribute();
    bool parseReference();
    bool processReference();

    bool parseExternalID();
    bool parsePEReference();
    bool parseMarkupdecl();
    bool parseAttlistDecl();
    bool parseAttType();
    bool parseAttValue();
    bool parseElementDecl();
    bool parseNotationDecl();
    bool parseChoiceSeq();
    bool parseEntityDecl();
    bool parseEntityValue();

    bool parseString();

    bool insertXmlRef( const QString&, const QString&, bool );

    bool reportEndEntities();
    void reportParseError( const QString& error );

    typedef bool (Qt3XmlSimpleReader::*ParseFunction) ();
    void unexpectedEof( ParseFunction where, int state );
    void parseFailed( ParseFunction where, int state );
    void pushParseState( ParseFunction function, int state );

    void setUndefEntityInAttrHack(bool b);

    friend class Qt3XmlSimpleReaderPrivate;
    friend class Qt3XmlSimpleReaderLocator;
    friend class QDomDocumentPrivate;
};

//
// SAX Locator
//

class QM_EXPORT_XML Qt3XmlLocator
{
public:
    Qt3XmlLocator();
    virtual ~Qt3XmlLocator();

    virtual int columnNumber() = 0;
    virtual int lineNumber() = 0;
//    QString getPublicId()
//    QString getSystemId()
};

//
// SAX handler classes
//

class QM_EXPORT_XML Qt3XmlContentHandler
{
public:
    virtual void setDocumentLocator( Qt3XmlLocator* locator ) = 0;
    virtual bool startDocument() = 0;
    virtual bool endDocument() = 0;
    virtual bool startPrefixMapping( const QString& prefix, const QString& uri ) = 0;
    virtual bool endPrefixMapping( const QString& prefix ) = 0;
    virtual bool startElement( const QString& namespaceURI, const QString& localName, const QString& qName, const Qt3XmlAttributes& atts ) = 0;
    virtual bool endElement( const QString& namespaceURI, const QString& localName, const QString& qName ) = 0;
    virtual bool characters( const QString& ch ) = 0;
    virtual bool ignorableWhitespace( const QString& ch ) = 0;
    virtual bool processingInstruction( const QString& target, const QString& data ) = 0;
    virtual bool skippedEntity( const QString& name ) = 0;
    virtual QString errorString() = 0;
};

class QM_EXPORT_XML Qt3XmlErrorHandler
{
public:
    virtual bool warning( const Qt3XmlParseException& exception ) = 0;
    virtual bool error( const Qt3XmlParseException& exception ) = 0;
    virtual bool fatalError( const Qt3XmlParseException& exception ) = 0;
    virtual QString errorString() = 0;
};

class QM_EXPORT_XML Qt3XmlDTDHandler
{
public:
    virtual bool notationDecl( const QString& name, const QString& publicId, const QString& systemId ) = 0;
    virtual bool unparsedEntityDecl( const QString& name, const QString& publicId, const QString& systemId, const QString& notationName ) = 0;
    virtual QString errorString() = 0;
};

class QM_EXPORT_XML Qt3XmlEntityResolver
{
public:
    virtual bool resolveEntity( const QString& publicId, const QString& systemId, Qt3XmlInputSource*& ret ) = 0;
    virtual QString errorString() = 0;
};

class QM_EXPORT_XML Qt3XmlLexicalHandler
{
public:
    virtual bool startDTD( const QString& name, const QString& publicId, const QString& systemId ) = 0;
    virtual bool endDTD() = 0;
    virtual bool startEntity( const QString& name ) = 0;
    virtual bool endEntity( const QString& name ) = 0;
    virtual bool startCDATA() = 0;
    virtual bool endCDATA() = 0;
    virtual bool comment( const QString& ch ) = 0;
    virtual QString errorString() = 0;
};

class QM_EXPORT_XML Qt3XmlDeclHandler
{
public:
    virtual bool attributeDecl( const QString& eName, const QString& aName, const QString& type, const QString& valueDefault, const QString& value ) = 0;
    virtual bool internalEntityDecl( const QString& name, const QString& value ) = 0;
    virtual bool externalEntityDecl( const QString& name, const QString& publicId, const QString& systemId ) = 0;
    virtual QString errorString() = 0;
};


class QM_EXPORT_XML Qt3XmlDefaultHandler : public Qt3XmlContentHandler, public Qt3XmlErrorHandler, public Qt3XmlDTDHandler, public Qt3XmlEntityResolver, public Qt3XmlLexicalHandler, public Qt3XmlDeclHandler
{
public:
    Qt3XmlDefaultHandler() { }
    virtual ~Qt3XmlDefaultHandler() { }

    void setDocumentLocator( Qt3XmlLocator* locator );
    bool startDocument();
    bool endDocument();
    bool startPrefixMapping( const QString& prefix, const QString& uri );
    bool endPrefixMapping( const QString& prefix );
    bool startElement( const QString& namespaceURI, const QString& localName, const QString& qName, const Qt3XmlAttributes& atts );
    bool endElement( const QString& namespaceURI, const QString& localName, const QString& qName );
    bool characters( const QString& ch );
    bool ignorableWhitespace( const QString& ch );
    bool processingInstruction( const QString& target, const QString& data );
    bool skippedEntity( const QString& name );

    bool warning( const Qt3XmlParseException& exception );
    bool error( const Qt3XmlParseException& exception );
    bool fatalError( const Qt3XmlParseException& exception );

    bool notationDecl( const QString& name, const QString& publicId, const QString& systemId );
    bool unparsedEntityDecl( const QString& name, const QString& publicId, const QString& systemId, const QString& notationName );

    bool resolveEntity( const QString& publicId, const QString& systemId, Qt3XmlInputSource*& ret );

    bool startDTD( const QString& name, const QString& publicId, const QString& systemId );
    bool endDTD();
    bool startEntity( const QString& name );
    bool endEntity( const QString& name );
    bool startCDATA();
    bool endCDATA();
    bool comment( const QString& ch );

    bool attributeDecl( const QString& eName, const QString& aName, const QString& type, const QString& valueDefault, const QString& value );
    bool internalEntityDecl( const QString& name, const QString& value );
    bool externalEntityDecl( const QString& name, const QString& publicId, const QString& systemId );

    QString errorString();

private:
    Qt3XmlDefaultHandlerPrivate *d;
};


//
// inlines
//

inline bool Qt3XmlSimpleReader::atEnd()
{ return (c.unicode()|0x0001) == 0xffff; }
inline int Qt3XmlAttributes::count() const
{ return length(); }


#endif //QT_NO_XML

#endif
