/*  This file is part of the KDE project
    Copyright (C) 2006 basysKom GmbH <info@basyskom.de>
    Copyright (C) 2006 ROAD GmbH <info@road-gmbh.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef SAFEDELETE_H
#define SAFEDELETE_H

#include<qobject.h>
#include<qobjectlist.h>

class SafeDelete;
class SafeDeleteLock
{
public:
	SafeDeleteLock(SafeDelete *sd);
	~SafeDeleteLock();

private:
	SafeDelete *_sd;
	bool own;
	friend class SafeDelete;
	void dying();
};

class SafeDelete
{
public:
	SafeDelete();
	~SafeDelete();

	void deleteLater(QObject *o);

	// same as QObject::deleteLater()
	static void deleteSingle(QObject *o);

private:
	QObjectList list;
	void deleteAll();

	friend class SafeDeleteLock;
	SafeDeleteLock *lock;
	void unlock();
};

class SafeDeleteLater : public QObject
{
	Q_OBJECT
public:
	static SafeDeleteLater *ensureExists();
	void deleteItLater(QObject *o);

private slots:
	void explode();

private:
	SafeDeleteLater();
	~SafeDeleteLater();

	QObjectList list;
	friend class SafeDelete;
	static SafeDeleteLater *self;
};

#endif
