#include <config.h>
#if defined(ENABLE_BOOKMARKS)
#include <klocale.h>

#include "bookmarkeditor.h"

#include <qframe.h>
#include <qheader.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

namespace
{
QPixmap getPngPixmap( const QString &name )
{
    return QPixmap( KGlobal::dirs()->findResource("icon",name+".png") );
}
}

/* 
 *  Constructs a BookmarkEditorBase which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
BookmarkEditorBase::BookmarkEditorBase( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    QPixmap image0 = getPngPixmap( "bookmark_goto" );
    QPixmap image1 = getPngPixmap( "bookmark_newfolder" );
    QPixmap image2 = getPngPixmap( "bookmark_new" );
    QPixmap image3 = getPngPixmap( "bookmark_edit" );
    QPixmap image4 = getPngPixmap( "bookmark_delete" );
    QPixmap image5 = getPngPixmap( "bookmark_up" );
    QPixmap image6 = getPngPixmap( "bookmark_down" );
    QPixmap image7 = getPngPixmap( "bookmark_left" );
    QPixmap image8 = getPngPixmap( "bookmark_right" );
    if ( !name )
	setName( "BookmarkEditorBase" );
    //resize( 279, 299 ); 
    setCaption( i18n( "Bookmarks" ) );
    BookmarkEditorBaseLayout = new QGridLayout( this ); 
    BookmarkEditorBaseLayout->setSpacing( 4 );
    BookmarkEditorBaseLayout->setMargin( 4 );

    Frame4 = new QFrame( this, "Frame4" );
    Frame4Layout = new QHBoxLayout( Frame4 );

    gotoBookmarkButton = new QToolButton( Frame4, "gotoBookmarkButton" );
    gotoBookmarkButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, gotoBookmarkButton->sizePolicy().hasHeightForWidth() ) );
    gotoBookmarkButton->setText( QString::null );
    gotoBookmarkButton->setPixmap( image0 );
    gotoBookmarkButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( gotoBookmarkButton );

    Line1_2 = new QFrame( Frame4, "Line1_2" );
    Line1_2->setFrameStyle( QFrame::VLine | QFrame::Sunken );
    Frame4Layout->addWidget( Line1_2 );

    newFolderButton = new QToolButton( Frame4, "newFolderButton" );
    newFolderButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, newFolderButton->sizePolicy().hasHeightForWidth() ) );
    newFolderButton->setText( QString::null );
    newFolderButton->setPixmap( image1 );
    newFolderButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( newFolderButton );

    newBookmarkButton = new QToolButton( Frame4, "newBookmarkButton" );
    newBookmarkButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, newBookmarkButton->sizePolicy().hasHeightForWidth() ) );
    newBookmarkButton->setText( QString::null );
    newBookmarkButton->setPixmap( image2 );
    newBookmarkButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( newBookmarkButton );

    editBookmarkButton = new QToolButton( Frame4, "editBookmarkButton" );
    editBookmarkButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, editBookmarkButton->sizePolicy().hasHeightForWidth() ) );
    editBookmarkButton->setText( QString::null );
    editBookmarkButton->setPixmap( image3 );
    editBookmarkButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( editBookmarkButton );

    deleteBookmarkButton = new QToolButton( Frame4, "deleteBookmarkButton" );
    deleteBookmarkButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, deleteBookmarkButton->sizePolicy().hasHeightForWidth() ) );
    deleteBookmarkButton->setText( QString::null );
    deleteBookmarkButton->setPixmap( image4 );
    deleteBookmarkButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( deleteBookmarkButton );

    Line1 = new QFrame( Frame4, "Line1" );
    Line1->setFrameStyle( QFrame::VLine | QFrame::Sunken );
    Frame4Layout->addWidget( Line1 );

    moveUpButton = new QToolButton( Frame4, "moveUpButton" );
    moveUpButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, moveUpButton->sizePolicy().hasHeightForWidth() ) );
    moveUpButton->setText( QString::null );
    moveUpButton->setPixmap( image5 );
    moveUpButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( moveUpButton );

    moveDownButton = new QToolButton( Frame4, "moveDownButton" );
    moveDownButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, moveDownButton->sizePolicy().hasHeightForWidth() ) );
    moveDownButton->setText( QString::null );
    moveDownButton->setPixmap( image6 );
    moveDownButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( moveDownButton );

    moveBookmarkOutOfFolderButton = new QToolButton( Frame4, "moveBookmarkOutOfFolderButton" );
    moveBookmarkOutOfFolderButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, moveBookmarkOutOfFolderButton->sizePolicy().hasHeightForWidth() ) );
    moveBookmarkOutOfFolderButton->setText( QString::null );
    moveBookmarkOutOfFolderButton->setPixmap( image7 );
    moveBookmarkOutOfFolderButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( moveBookmarkOutOfFolderButton );

    moveBookmarkIntoFolderButton = new QToolButton( Frame4, "moveBookmarkIntoFolderButton" );
    moveBookmarkIntoFolderButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, moveBookmarkIntoFolderButton->sizePolicy().hasHeightForWidth() ) );
    moveBookmarkIntoFolderButton->setText( QString::null );
    moveBookmarkIntoFolderButton->setPixmap( image8 );
    moveBookmarkIntoFolderButton->setAutoRaise( TRUE );
    Frame4Layout->addWidget( moveBookmarkIntoFolderButton );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Frame4Layout->addItem( spacer );

    BookmarkEditorBaseLayout->addMultiCellWidget( Frame4, 0, 0, 0, 2 );

    okButton = new QPushButton( this, "okButton" );
    okButton->setText( i18n( "&OK" ) );

    cancelButton = new QPushButton( this, "cancelButton" );
    cancelButton->setText( i18n( "&Cancel" ) );

#ifdef _QT_QPE_
    okButton->hide();
    cancelButton->hide();
#else
    BookmarkEditorBaseLayout->addWidget( okButton, 2, 1 );
    BookmarkEditorBaseLayout->addWidget( cancelButton, 2, 2 );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    BookmarkEditorBaseLayout->addItem( spacer_2, 2, 0 );
#endif

    view = new QListView( this, "view" );
    view->addColumn( i18n( "Title" ) );
    view->addColumn( i18n( "Location" ) );

    BookmarkEditorBaseLayout->addMultiCellWidget( view, 1, 1, 0, 2 );

    // signals and slots connections
    connect( okButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( view, SIGNAL( selectionChanged() ), this, SLOT( updateButtons() ) );
    connect( newBookmarkButton, SIGNAL( clicked() ), this, SLOT( newBookmark() ) );
    connect( editBookmarkButton, SIGNAL( clicked() ), this, SLOT( editBookmark() ) );
    connect( deleteBookmarkButton, SIGNAL( clicked() ), this, SLOT( deleteBookmark() ) );
    connect( moveUpButton, SIGNAL( clicked() ), this, SLOT( moveBookmarkUp() ) );
    connect( moveDownButton, SIGNAL( clicked() ), this, SLOT( moveBookmarkDown() ) );
    connect( gotoBookmarkButton, SIGNAL( clicked() ), this, SLOT( gotoBookmark() ) );
    connect( newFolderButton, SIGNAL( clicked() ), this, SLOT( newFolder() ) );
    connect( moveBookmarkOutOfFolderButton, SIGNAL( clicked() ), this, SLOT( moveBookmarkOutOfFolder() ) );
    connect( moveBookmarkIntoFolderButton, SIGNAL( clicked() ), this, SLOT( moveBookmarkIntoFolder() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
BookmarkEditorBase::~BookmarkEditorBase()
{
    // no need to delete child widgets, Qt does it all for us
}

void BookmarkEditorBase::deleteBookmark()
{
    qWarning( "BookmarkEditorBase::deleteBookmark(): Not implemented yet!" );
}

void BookmarkEditorBase::editBookmark()
{
    qWarning( "BookmarkEditorBase::editBookmark(): Not implemented yet!" );
}

void BookmarkEditorBase::gotoBookmark()
{
    qWarning( "BookmarkEditorBase::gotoBookmark(): Not implemented yet!" );
}

void BookmarkEditorBase::moveBookmarkDown()
{
    qWarning( "BookmarkEditorBase::moveBookmarkDown(): Not implemented yet!" );
}

void BookmarkEditorBase::moveBookmarkIntoFolder()
{
    qWarning( "BookmarkEditorBase::moveBookmarkIntoFolder(): Not implemented yet!" );
}

void BookmarkEditorBase::moveBookmarkOutOfFolder()
{
    qWarning( "BookmarkEditorBase::moveBookmarkOutOfFolder(): Not implemented yet!" );
}

void BookmarkEditorBase::moveBookmarkUp()
{
    qWarning( "BookmarkEditorBase::moveBookmarkUp(): Not implemented yet!" );
}

void BookmarkEditorBase::newBookmark()
{
    qWarning( "BookmarkEditorBase::newBookmark(): Not implemented yet!" );
}

void BookmarkEditorBase::newFolder()
{
    qWarning( "BookmarkEditorBase::newFolder(): Not implemented yet!" );
}

void BookmarkEditorBase::updateButtons()
{
    qWarning( "BookmarkEditorBase::updateButtons(): Not implemented yet!" );
}

#include "bookmarkeditor.moc"
#endif
