/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifdef KONQ_GUI_ROAD

#include "mainwindow_road.h"
#include "popupaction.h"


#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qlineedit.h>
#include <qstatusbar.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qhbuttongroup.h>
#include <qaccel.h>
#include <qhbox.h>

#ifdef _QT_QPE_
#include <qtopia/qcopenvelope_qws.h>
#endif

#include <roadmode.h>
#include <roadtabbar.h>
#include <websearcher.h>
#include <toggletoolbutton.h>
#include <urlcombo.h>

#include <klocale.h>
#include <kstdaction.h>
#include <kfiledialog.h>

#include "khtml_part.h"

CreateAction::CreateAction( QWidget *parent, QWidget *addToWidget, KActionCollection *actionCollection)
    : m_parent( parent ),
      m_addToWidget( addToWidget ),
      m_actionCollection( actionCollection )
{}

KAction *CreateAction::operator() ( const char *text, int accel, const char *name, bool toggle )
{
    return activate( new QAction( QString::null, i18n( text ),
                           accel,
                           m_parent, name, toggle ) );
}

KAction *CreateAction::operator() ( const QIconSet &icon, int accel, const char *name, bool toggle )
{
    return activate( new QAction( QString::null, icon, QString::null,
                           accel,
                           m_parent, name, toggle ) );
}

KAction *CreateAction::activate( QAction *qa )
{
    qa->addTo( m_addToWidget );
    KAction *kAct = m_actionCollection->action( qa->name() );
    if (kAct)
    {
        kAct->setEnabled( true );
    }
    return kAct;
}

////////////////////////////////////////////////////////////////////////////////////

MainWindowRoad::MainWindowRoad()
    : MainWindowBase( 0, 0, WType_TopLevel|WDestructiveClose ),
      m_defaultModeToolBarIdx( -1 ),
      m_defaultModeIdx( -1 ),
      m_webSearcher( 0 ),
      m_searchBackward( 0 ),
      m_zoomIn( 0 ),
      m_zoomOut( 0 ),
      m_showSecurity( 0 ),
      m_roadcfg("roadrc"),
      m_closeTab( 0 ),
      m_tabCountBtn( 0 ),
      m_mmCloseView( 0 )
{
   KConfig *config = KGlobal::config();
   KConfigGroupSaver saver( config, QString::fromLatin1( "General" ) );

   m_securityIcons[NotCrypted] = getPngPixmap( "decrypted" );
   m_securityIcons[Encrypted] = getPngPixmap( "encrypted" );
   m_securityIcons[Mixed] = getPngPixmap( "halfencrypted" );

   s_winStyle = STYLE_MAXIMIZED;

#ifdef _QT_QPE_
   connect(qApp, SIGNAL(appMessage(const QCString &, const QByteArray &)),
            this, SLOT(message(const QCString &, const QByteArray &)) );
#endif
}

MainWindowRoad::~MainWindowRoad()
{
}

void MainWindowRoad::createNewWindow( const KURL &url, const KParts::URLArgs &args,
				    const KParts::WindowArgs &windowArgs,
				    KParts::ReadOnlyPart *&part )
{
    BrowserView *view = createNewView( url, args, windowArgs, part );
    view->setOpenedByJS( true );
    updateTabCountDisplay();
}


BrowserView *MainWindowRoad::createNewViewInternal(const KURL &url,
	const KParts::URLArgs &args, bool /*withStatus*/)
{
    BrowserView *v = MainWindowBase::createNewViewInternal(url, args);
#if QT_VERSION < 0x030000
    v->document()->view()->setVScrollBarMode(QScrollView::Auto);
    v->document()->view()->setHScrollBarMode(QScrollView::Auto);
#else
    v->document()->view()->setVScrollBarMode(QScrollView::Auto);
    v->document()->view()->setHScrollBarMode(QScrollView::Auto);
#endif 
    updateTabCountDisplay();
    return v;
}

void MainWindowRoad::setDocument( const QString &document )
{
#ifdef _QT_QPE_
    DocLnk lnk( document );
    createNewView( lnk.file().prepend( "file:" ) );
#else
//    QMainWindow::setDocument( document );
#endif
}


KURL MainWindowRoad::startupURL() const
{
    return BrowserView::homeURL();
}

QPtrListIterator<RoadMode> MainWindowRoad::getModeIterator()
{
    return QPtrListIterator<RoadMode>( m_modes );
}

int MainWindowRoad::getDefaultMode() const
{
    return m_defaultModeIdx;
}

void MainWindowRoad::setDefaultMode( int modeIdx )
{
    // shift the mode bars around to reflect the new default mode:
    // only the default bar doesn't get its own line.
    QPtrListIterator<RoadMode> it = getModeIterator();
    it.toFirst();
    for (int i = 0; it.current(); ++it, ++i)
    {
        const bool isDefault = (i == modeIdx );
        it.current()->setDefault( isDefault, m_defaultModeToolBarIdx );
    }
    m_defaultModeIdx = modeIdx;
}

void MainWindowRoad::defaultLook()
{
    // HACK: calling showMaximized after having been in fullscreen mode
    //       doesn't restore the window correctly, so we showNormal in between.
    if ( STYLE_MAXIMIZED == s_winStyle )
        showNormal();
    MainWindowBase::defaultLook();
}

void MainWindowRoad::initGUI()
{
    setToolBarsMovable( false );
    initMainToolbar( );
    initModeSelector( );
    disableUnusedComponents();
}

void MainWindowRoad::addNewView( View *view )
{
    MainWindowBase::addNewView( view );
    emit announceNewView( view );
    stopEnabled( false );

    updateTabCountDisplay();
}

void MainWindowRoad::disconnectView( View *view )
{
    KAction *kAct = view->actionCollection()->action( "go_stop" );
    if (kAct)
        disconnect( kAct, SIGNAL( enabled( bool ) ), this, SLOT( stopEnabled( bool ) ) );
    connect( m_searchBackward, SIGNAL( clicked() ), this, SLOT( findPrev() ) );

    MainWindowBase::disconnectView( view );
}

void MainWindowRoad::connectView( View *view )
{
    MainWindowBase::connectView( view );

    KAction *kAct = view->actionCollection()->action( "go_stop" );
    if (kAct)
        connect( kAct, SIGNAL( enabled( bool ) ), this, SLOT( stopEnabled( bool ) ) );
    connect( m_searchBackward, SIGNAL( clicked() ), this, SLOT( findPrev() ) );
    BrowserView *bView = view->isBrowser();
    if ( bView )
    {
        connect( bView->document()->browserExtension(), SIGNAL( setPageSecurity( int ) ),
                 this, SLOT( showSecurity( int ) ) );
        showSecurity( bView->getPageSecurity() );
    }

    emit( viewConnected( view ) );
}

void MainWindowRoad::showSecurity( int level )
{
    static const QColor secureSiteBackground( 255, 250, 197 );

    m_showSecurity->setPixmap( ( 0 <= level && NumSecurityLevels > level )
             ? m_securityIcons[level]
             : QPixmap() );

    QPalette palette = this->palette();
    if ( Encrypted == level )
        palette.setColor( QColorGroup::Base, secureSiteBackground );
    m_locationCombo->setPalette( palette );
}

void MainWindowRoad::showCertificate()
{
    if (m_currentView)
    {
        KAction *kAct = m_currentView->part()->actionCollection()->action( "security" );
        if (kAct)
            kAct->activate();
    }
}

void MainWindowRoad::setupView( View *view )
{
    MainWindowBase::setupView( view );

    KAction *kAct = view->actionCollection()->action( "go_stop" );
    if (kAct)
        stopEnabled( kAct->isEnabled() );
}

void MainWindowRoad::statusMessage(const QString&, bool)
{
    // ignore the message
}

void MainWindowRoad::viewWidgetDestroyed()
{
    QWidget *widget = const_cast<QWidget *>( static_cast<const QWidget *>( sender() ) );
    if ( widget )
        emit announceViewWidgetDestroyed( m_widgetStack->id( widget ) );
    MainWindowBase::viewWidgetDestroyed();

    updateTabCountDisplay();
}

void MainWindowRoad::updateTabCountDisplay()
{
/*
    int validTabs=0;
    QPtrListIterator<View> it(m_views);
    for(;it.current();++it)
    {
        if (!static_cast<BrowserView*>(*it)->openedByJS())
            validTabs++;
    }*/
    if (!m_tabCountBtn)
        return;
    if ( m_views.count() <= 1 )
        m_tabCountBtn->setTextLabel( QString::null );
    else if ( m_views.count() > 99 )
        m_tabCountBtn->setTextLabel( tr("##") );
    else
    {
        QString numOpenTabs;
        numOpenTabs.setNum( m_views.count() );
        m_tabCountBtn->setTextLabel( numOpenTabs );
    }
    m_tabCountBtn->update();
/*
    if ( validTabs <= 1 )
    {
        if ( m_closeTab )
            m_closeTab->setEnabled(false);
        if (m_mmCloseView)
            m_mmCloseView->setEnabled( false );
    }
    else
    {
        if ( m_closeTab )
            m_closeTab->setEnabled(true);
        if (m_mmCloseView)
            m_mmCloseView->setEnabled( true );
    }
*/
//    updateTabCloseButton();
}

void MainWindowRoad::closeView( View *view )
{
kdDebug() << "MainWindowRoad::closeView( View *view )" << endl;
    // prevent the final tab from being closed
    if ( m_views.count() > 1 )
    {
        MainWindowBase::closeView( view );
    }
    updateTabCountDisplay();
}

void MainWindowRoad::showPreferences()
{
   // turn off the action so the dialog can't be started twice
   KAction *prefsAction = actionCollection()->action( "options_configure" );
   bool prefsWasEnabled = false;
   if (prefsAction)
   {
       prefsWasEnabled = prefsAction->isEnabled();
       prefsAction->setEnabled(false);
   }
   // preferences window manipulates the same config file as we do, so clean up before calling it:
   m_roadcfg.sync();
   MainWindowBase::showPreferences();
   m_roadcfg.reparseConfiguration();
   // turn the action back on if necessary
   if (prefsAction)
       prefsAction->setEnabled(prefsWasEnabled);
}

void MainWindowRoad::toggleFullScreen( bool isFullScreen )
{
    if ( isFullScreen )
	emit fullscreen();
    actionCollection()->action( "bookmark_edit" )->setEnabled( !isFullScreen );
    actionCollection()->action( "options_configure" )->setEnabled( !isFullScreen );
    MainWindowBase::toggleFullScreen( isFullScreen );
}

void MainWindowRoad::setLocationBarURL( const QString &url )
{
    if ( !m_locationCombo )
        return;

    bool exists = false;
    int i = 0;
    if ( m_locationCombo->currentText().isEmpty() )
        m_locationCombo->removeItem( m_locationCombo->currentItem() );
    for (; i < m_locationCombo->count(); ++i )
        if ( m_locationCombo->text( i ) == url )
        {
            exists = true;
            break;
        }

    if ( exists )
        m_locationCombo->setCurrentItem( i );
    else
    {
        URLCombo* filteringBox = dynamic_cast<URLCombo*>(m_locationCombo);
        if (filteringBox)
             filteringBox->addURL( url );
        else
        {
            m_locationCombo->insertItem( url, 0 );
            m_locationCombo->setCurrentItem( 0 );
        }
    }
}

void MainWindowRoad::initMainToolbar( )
{
    QToolBar *tb = new QToolBar( this, "mainToolbar" );

    QAction *qa;

    qa = new PopupAction( initMainMenu( this ), QString::null,
                          getPng( "menu_road" ), QString::null, 
                          CTRL + Key_Down, this );
    qa->addTo( tb );

    m_zoomIn = new QToolButton( getPng( "zoom_in" ), QString::null,
                         QString::null, this, SLOT( zoomIn() ), tb, "zoomIn" );
    m_zoomOut = new QToolButton( getPng( "zoom_out" ), QString::null,
                         QString::null, this, SLOT( zoomOut() ), tb, "zoomOut" );
    connect( this, SIGNAL( zoomValueSelected( int ) ),
             this, SLOT( validateZoomButtons( int ) ) );

    CreateAction createAction( m_changeable, tb, actionCollection() );
    createAction( getPng( "back" ), 0/*Qt::ALT+Qt::Key_Left*/, "go_back" );
    createAction( getPng( "forward" ), 0/*Qt::ALT+Qt::Key_Right*/, "go_forward" );
    createAction( getPng( "reload" ), 0/*Qt::Key_F5*/, "view_redisplay" );
    createAction( getPng( "stop" ), 0/*Qt::Key_Escape*/, "go_stop" );
    createAction( getPng( "images_display" ), 0/*Qt::CTRL+Qt::Key_I*/, "view_load_images" );

    tb->setHorizontalStretchable( false );
}

QPopupMenu* MainWindowRoad::initMainMenu( QWidget* parent )
{
    QPopupMenu *mm = new QPopupMenu( parent );
    CreateAction createAction( m_changeable, mm, actionCollection() );

    createAction( "&New tab", 0/*Qt::CTRL+Qt::Key_N*/, "file_new" );
    m_mmCloseView = createAction( "&Close tab", 0/*Qt::CTRL+Qt::Key_W*/, "view_close" );
    if (m_mmCloseView)
        m_mmCloseView->setEnabled( false );
    mm->insertSeparator();

    QAction *qa= new PopupAction( m_zoomMenu, QString::null, QIconSet(),
                            i18n( "&Zoom" ), 0, this );
    qa->addTo( mm );
    if ( s_winStyle != STYLE_FULLSCREEN )
        createAction( "Full&screen", 0/*Qt::Key_F11*/, "view_fullscreen", true);
    createAction( "Autoload Images", 0/*Qt::CTRL+Qt::Key_I*/, "view_autoload_images", true );

    mm->insertSeparator();

    createAction( "&Home Page",0/* Qt::ALT+Qt::Key_Home*/, "go_home" );

    mm->insertSeparator();

#if defined(ENABLE_BOOKMARKS)
    if ( m_bookmarks )
    {
        createAction( "&Add Bookmark", 0/*Qt::CTRL+Qt::Key_A*/, "bookmark_add" );
        createAction( "&Bookmarks...", 0/*Qt::CTRL+Qt::Key_B*/, "bookmark_edit" );
        mm->insertSeparator();
    }
#endif

    createAction( "&Preferences...", 0/*Qt::CTRL+Qt::Key_P*/, "options_configure" );

    mm->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Maximum ) );

    return mm;
}

void MainWindowRoad::disableUnusedComponents()
{
    // Disable unused components
    delete m_closeFind;
    m_closeFind = 0;
    disconnect(m_statusBarAutoHideTimer, SIGNAL(timeout()),
               statusBar(), SLOT(hide()));
    statusBar()->hide();
}

// helper for initModeSelector (see below)
namespace
{
int getToolBarIdx( QMainWindow *win, QToolBar *bar )
{
//#if (QT_VERSION < 0x030000)
    QMainWindow::ToolBarDock dock;
//#else
//    Dock dock;
//#endif
    int idx;
    bool nl;
    int extraOffset;
    win->getLocation( bar, dock, idx, nl, extraOffset );
    return idx;
}
}
/* Three widgets have a special group relationship:
    -URL bar (user can type in a URL and open that site in the current tab)
    -search bar (user can type in terms and search for them in one of
      several search engines, or within the current site)
    -tab bar (displays all open sites as tabs and allows switching between and
      closing them)

   One of these widgets is the default, and is always visible.
   Each of the others is associated with a toggle button that hides or shows it.
   When shown, non-default widgets appear as an additional bar below the default widget.
   Non-default widgets may also choose to hide themselves automatically.

   All this is managed by four toolbars: one for the buttons, one for each bar. */
void MainWindowRoad::initModeSelector()
{
    QToolBar *modeSel = new QToolBar( this, "modeSelToolbar" );
    // the default toolbar will always be the first of the mode toolbars.
    // Note which toolbar index that is so we can move the default there as required.
    m_defaultModeToolBarIdx = getToolBarIdx( this, modeSel ) + 1;

    initUrlMode( modeSel );
    initSearchMode( modeSel );
    initTabMode( modeSel );

    m_roadcfg.setGroup("Default Bar");
    setDefaultMode( m_roadcfg.readNumEntry( "Index", 0 ) );
}

void MainWindowRoad::initUrlMode( QToolBar *modeSel )
{
    ModeBar *modeBar = addMode( new ToggleToolButton( getPng( "www" ), modeSel ), "URL Entry", "urlTB" );
    m_locationCombo = new URLCombo( modeBar, "locationcombo" );
    m_locationCombo->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred) );
    m_locationCombo->setFocus(); // ensure that user can type URL immediately
    m_showSecurity = new QToolButton( getPng( "decrypted" ), QString::null,
                         QString::null, this, SLOT( showCertificate() ), modeBar, "showCertificate" );
    modeBar->finalize();
    connect( this, SIGNAL( clearURLHistoryClicked() ), m_locationCombo, SLOT( clearURLHistory() ) );
    connect( m_locationCombo, SIGNAL( activated( const QString & ) ),
             this, SLOT( focusView() ) );
    connect( m_locationCombo, SIGNAL( activated( const QString & ) ),
             this, SLOT( urlEntered( const QString & ) ) );
    connect( m_locationCombo, SIGNAL( activated( const QString & ) ),
             modeBar, SLOT( autoHide() ) );
    connect( modeBar, SIGNAL( autoHidden() ), 
             this, SLOT( focusView() ) );
}

void MainWindowRoad::updateTabCloseButton(int index)
{
    View *view = findViewForWidget( index );
    if (!view) 
       return;

    int validTabs=0;
    QPtrListIterator<View> it(m_views);
    for(;it.current();++it)
    {
        if (!static_cast<BrowserView*>(*it)->openedByJS())
            validTabs++;
    }

    bool enable = false;
    if (dynamic_cast<BrowserView*>(view)->openedByJS())
        enable = true;
    else if ( validTabs > 1 )
        enable = true;

    if ( !enable )
    {
        if ( m_closeTab )
            m_closeTab->setEnabled(false);
        if (m_mmCloseView)
            m_mmCloseView->setEnabled( false );
    }
    else
    {
        if ( m_closeTab )
            m_closeTab->setEnabled(true);
        if (m_mmCloseView)
            m_mmCloseView->setEnabled( true );
    }
}

// Search mode, consisting of
//  a text entry field which accepts search terms,
//  and a combo box for selecting the search engine or "search in page".
void MainWindowRoad::initSearchMode( QToolBar *modeSel )
{
    ToggleToolButton* tt = new ToggleToolButton( getPng( "find" ), modeSel );
    ModeBar *modeBar = addMode( tt, "Search (Web or in Page)", "searchTB", true );
    QWidget *searchWidget = new QHBox( modeBar );

    QComboBox *searchModeCombo = new QComboBox( false, searchWidget, "engine_select" );
    m_findInput = new QLineEdit( searchWidget, "pageSearch" );
    m_webSearcher = new WebSearcher( modeBar, "webSearcher" );

    m_searchBackward =
        new QToolButton( getPng( "search_backward" ), QString::null,
                         QString::null, 0, 0, modeBar, "searchBackward" );
    new QToolButton( getPng( "search_forward" ), QString::null,
                     QString::null, m_findInput, SIGNAL(returnPressed()),
                     modeBar, "searchForward" );

    searchModeCombo->insertItem( getPngPixmap( "filefind" ) ); // file search must come first if activateSearchMode() is to work
    for (uint i = 0; i < m_webSearcher->getNumEngines(); ++i)
    {
        if (m_webSearcher->getEngineData(i))
            searchModeCombo->insertItem( m_webSearcher->getEngineData(i)->pixmap, "text" );
    }

    m_roadcfg.setGroup("Default Search");
    searchModeCombo->setCurrentItem( m_roadcfg.readNumEntry( "Index", 0 ) );
    activateSearchMode( searchModeCombo->currentItem() );
    modeBar->finalize();
    connect( searchModeCombo, SIGNAL( activated( int ) ), this, SLOT( activateSearchMode( int ) ) );
    connect( this, SIGNAL( viewConnected( View * ) ), m_webSearcher, SLOT( connectView( View * ) ) );
    connect( modeBar, SIGNAL( autoHidden() ), this, SLOT( focusView() ) );
    connect( tt, SIGNAL( toggledOn() ), m_findInput, SLOT( setFocus() ) );
}

void MainWindowRoad::initTabMode( QToolBar *modeSel )
{
    m_tabCountBtn = new OverlayLabelButton( getPng( "show_tabs" ), modeSel );
    ModeBar *modeBar = addMode( m_tabCountBtn, "Tabbed Browsing", "tabTB" );

    QToolButton *newTab = new QToolButton( getPng( "new_tab" ), QString::null, QString::null, 
                                           this, SLOT( newView() ), modeBar );
    newTab->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
    RoadTabBar* tabBar = new RoadTabBar( QIconSet(), getPng( "loading" ),
                                         modeBar, "tabbar" );
    m_closeTab = new QToolButton( getPng( "close_tab" ), QString::null, QString::null, 
                                  this, SLOT( closeView() ), modeBar );
    m_closeTab->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
    m_closeTab->setEnabled(false);
    modeBar->finalize();
    // The following connects yoke RoadTabBar and m_widgetStack together.
    // It may make more sense to connect tabBar and m_views instead, but this is awkward
    //  because the indices of the views change whenever one is removed,
    // which isn't true of the stack and the tab bar.
    connect( this, SIGNAL( announceNewView( View * ) ), tabBar, SLOT( newView( View * ) ) );
    connect( tabBar, SIGNAL( selectedNotShow( int ) ), this, SLOT( activateViewWidget( int ) ) );
    connect( m_widgetStack, SIGNAL( aboutToShow( int ) ), tabBar, SLOT( setCurrentTab( int ) ) );
    connect( this, SIGNAL( announceViewWidgetDestroyed( int ) ), tabBar, SLOT( removeTab( int ) ) );
    connect( tabBar, SIGNAL( selectedNotShow( int ) ), modeBar, SLOT( autoHide() ) );
    connect( tabBar, SIGNAL( closed( int ) ), this, SLOT( closeViewWidget( int ) ) );
    connect( tabBar, SIGNAL( requestNewView() ), this, SLOT( newView() ) );
    connect( tabBar, SIGNAL( selected( int ) ), this, SLOT( updateTabCloseButton( int ) ) ); 
    connect( m_widgetStack, SIGNAL( aboutToShow( int ) ), tabBar, SLOT( updateTabCloseButton( int ) ) );
    connect( modeBar, SIGNAL( autoHidden() ), this, SLOT( focusView() ) );
}

ModeBar *MainWindowRoad::addMode( ToggleToolButton *btn, const char *desc,
                                  const char *modeBarName, bool dockBottom )
{
    ModeBar *modeBar = new ModeBar( getPng( "pinup" ), getPng( "pindown" ), this, modeBarName );
    m_modes.append( new RoadMode( modeBar, btn, desc, dockBottom ) );
    connect( this, SIGNAL( fullscreen() ), modeBar, SLOT( autoHide() ) );
    return modeBar;
}

void MainWindowRoad::activateSearchMode( int i )
{
    m_findInput->disconnect( SIGNAL( returnPressed() ) );
    if (0 >= i)
    {
        connectFindInPage( m_currentView );
        if (m_searchBackward)
            m_searchBackward->show();
    }
    else if (m_webSearcher)
    {
        m_webSearcher->activate( m_findInput, static_cast<uint>(i) - 1 );
        if (m_searchBackward)
            m_searchBackward->hide();
    }

    // make sure the current setting remains on next startup
    m_roadcfg.setGroup ("Default Search");
    m_roadcfg.writeEntry( "Index",  i );
}

void MainWindowRoad::activateViewWidget( int idx )
{
    View *v = findViewForWidget( idx );
    if (v)
        setActiveView( v );
}

void MainWindowRoad::closeViewWidget( int idx )
{
    View *v = findViewForWidget( idx );
    if (v)
        closeView( v );
}

View *MainWindowRoad::findViewForWidget( int idx )
{
    // there's no direct connect from widget to view, so
    // we have to do a linear search.  This shouldn't
    // matter, since the number of views is small.
    QWidget *soughtWidget = m_widgetStack->widget( idx );
    if (soughtWidget)
    {
        for ( QPtrListIterator<View> it( m_views ); it.current(); ++it )
        {
            if ( it.current()->part()->widget() == soughtWidget )
            {
                 return *it;
            }
        }
    }
    return 0;
}

// called when the "stop" button is enabled or disabled
// (that is, when a page starts or stops loading)
// To save space, we only show the stop or the refresh button, not both
void MainWindowRoad::stopEnabled( bool b )
{
    setActionVisible( "go_stop", b );
    setActionVisible( "view_redisplay", !b );
    
    m_modes.at(m_defaultModeIdx)->modeBar()->setThrobble(b);
}

void MainWindowRoad::zoomIn()
{
    doZoom( ZOOM_STEP );
}

void MainWindowRoad::zoomOut()
{
    doZoom( -ZOOM_STEP );
}

void MainWindowRoad::doZoom( int step )
{
    if (m_currentView)
    {
        const int newZoom = m_currentView->zoomValue() + step;
        if (canZoomTo( newZoom ))
            emit zoomValueSelected( newZoom );
    }
}

bool MainWindowRoad::canZoomTo( int newZoom ) const
{
    return (MIN_ZOOM <= newZoom && newZoom <= MAX_ZOOM);
}


void MainWindowRoad::validateZoomButtons( int zoomFactor )
{
    m_zoomIn->setEnabled( canZoomTo( zoomFactor + ZOOM_STEP ) );
    m_zoomOut->setEnabled( canZoomTo( zoomFactor - ZOOM_STEP ) );
}

QIconSet MainWindowRoad::getPng( const QString &name )
{
    return QIconSet( KGlobal::dirs()->findResource("icon",name+".png") );
}

QPixmap MainWindowRoad::getPngPixmap( const QString &name )
{
    return QPixmap( KGlobal::dirs()->findResource("icon",name+".png") );
}

void MainWindowRoad::message(const QCString &msg, const QByteArray &data)
{
/*
    qDebug("MainWindowRoad::message %s", (const char*)msg);
    QDataStream stream( data, IO_ReadOnly );
    if (msg == "setDocument(QString)" )
    {
        
        dat
        setDocument(QString);
    }
    else if (msg == "quit()" )
    {
        qApp->quit();
    }
*/
}



#include "mainwindow_road.moc"

#endif // !defined(KONQ_GUI_KIOSK) && !defined(KONQ_GUI_TV)

/*
 * vim:ts=8:et:sw=4
 */
