/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef __mainwindow_road_h__
#define __mainwindow_road_h__

#include <config.h>

#if !defined(KONQ_GUI_KIOSK)

#include "mainwindowbase.h"

class RoadMode;
class ModeBar;
class WebSearcher;
class OverlayLabelButton;
class ToggleToolButton;

class MainWindowRoad : public MainWindowBase
{
    Q_OBJECT
public:
    MainWindowRoad();
    ~MainWindowRoad();
    virtual KURL startupURL() const;
    QPtrListIterator<RoadMode> getModeIterator();
    int getDefaultMode() const;
signals:
    void announceNewView( View * );
    void announceViewWidgetDestroyed( int viewIdx );
    void viewConnected( View * );
    void fullscreen();
    void clearURLHistoryClicked();
public slots:
    void setDefaultMode( int modeIdx );
    void setDocument( const QString &s );
    void createNewWindow( const KURL &url, const KParts::URLArgs &args,
                        const KParts::WindowArgs &windowArgs, KParts::ReadOnlyPart *&part );

    /** receives and handles QCop messages */
    void message(const QCString &msg, const QByteArray &data);

protected:
    virtual BrowserView *createNewViewInternal( const KURL &url, 
		    const KParts::URLArgs &args, bool withStatus = true );

    virtual void defaultLook();
    virtual void initGUI();
    virtual void addNewView( View *view );
    virtual void disconnectView( View *view );
    virtual void connectView( View *view );
    virtual void setupView( View *view );
    virtual void closeView( View *view );

protected slots:
    virtual void statusMessage( const QString &msg, bool visible );
    virtual void viewWidgetDestroyed();
    virtual void showPreferences();
    virtual void toggleFullScreen( bool isFullScreen );
    virtual void setLocationBarURL( const QString &url );
private slots:
    void activateViewWidget( int idx );
    void stopEnabled( bool b );
    void activateSearchMode( int i );
    void closeViewWidget( int i );
    void showSecurity( int securityLevel );
    void showCertificate();
    void zoomIn();
    void zoomOut();
    void validateZoomButtons( int zoomFactor );
    void updateTabCloseButton( int index );
private:
    void initMainToolbar();
    void disableUnusedComponents();
    QPopupMenu *initMainMenu( QWidget *parent );
    void initModeSelector();
    ModeBar *addMode( ToggleToolButton *btn, const char *desc,
                      const char *modeBarName = 0, bool dockBottom = false );
    void initUrlMode( QToolBar *modeSel );
    void initSearchMode( QToolBar *modeSel );
    void initTabMode( QToolBar *modeSel );
    // convert a widget index into a view index
    View *findViewForWidget( int idx );
    void doZoom( int step );
    bool canZoomTo( int newZoomFactor ) const;
    void updateTabCountDisplay();
    QIconSet getPng( const QString &name );
    QPixmap getPngPixmap( const QString &name );

    // One of the following toolbars is the default, and is shown as part
    // of the main toolbar.  The others are hidden by default, and each can
    // appear temporarily in its own row below the main toolbar.  We must 
    // store them so we can dynamically change which is the default.
    QPtrList<RoadMode> m_modes;
    int m_defaultModeToolBarIdx;
    int m_defaultModeIdx;
    WebSearcher *m_webSearcher;
    QToolButton *m_searchBackward;
    QToolButton *m_zoomIn;
    QToolButton *m_zoomOut;
    QToolButton *m_showSecurity;
    KConfig m_roadcfg;
    QToolButton *m_closeTab;
    OverlayLabelButton *m_tabCountBtn;
    KAction *m_mmCloseView;

    // SecurityLevels correspond to the (unfortunately protected) KHTMLPart enum
    enum SecurityLevels {
        NotCrypted,
        Encrypted,
        Mixed,
        NumSecurityLevels
    };
    QPixmap m_securityIcons[NumSecurityLevels];
};

// Just a helper function object to minimize redundant code
class CreateAction
{
public:
    CreateAction( QWidget *parent, QWidget *addToWidget, KActionCollection *actionCollection);
    KAction *operator() ( const char *text, int accel, const char *name, bool toggle = false );
    KAction *operator() ( const QIconSet &icon, int accel, const char *name, bool toggle = false );
private:
    KAction *activate( QAction * );
    QWidget *m_parent;
    QWidget *m_addToWidget;
    KActionCollection *m_actionCollection;
};


#endif

#endif // __mainwindow_road_h__
