/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    As a special exception this program may be linked with Qt non-commercial 
    edition, the resulting executable be distributed, without including the 
    source code for the Qt non-commercial edition in the source distribution.

*/

#include <config.h>

#ifdef KONQ_GUI_ROAD
#include "preferences_road.h"
#include "roadmode.h"

#include <qcheckbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qhgroupbox.h>
#include <qvgroupbox.h>
#include <qvbuttongroup.h>
#include <qradiobutton.h>

#include <ksslsigners.h>
#include "ksslx509map.h"
#include "ksslcertchain.h"
#include "../dropin/kssl/ksslinfodlg.h"
#include "ksslx509v3.h"
#include "kfiledialog.h"
#include "kopenssl.h"
#include "kmdcodec.h"

#ifdef _QT_QPE_
#include <qtopia/qpeapplication.h>
#endif

#define DEFAULT_HOMEPAGE	"http://www.road-gmbh.de/"

namespace prAnon // anonymous namespaces aren't working correctly
{
int cookieAdviceToIndex( const QString &_advice )
{
    QString advice = _advice.lower();
    if ( advice == "reject" )
        return 1;
    if ( advice == "accept" )
        return 2;
    return 0;
}

QString indexToCookieAdvice( int idx )
{
    if ( idx == 1 )
       return QString::fromLatin1( "Reject" );
    if ( idx == 2 )
       return QString::fromLatin1( "Accept" );
   return QString::fromLatin1( "Ask" );
}
int UAToIndex( const QString &_advice )
{
    QString advice = _advice.lower();
    if ( advice == "safari" )
        return 1;
    if ( advice == "ie" )
        return 2;
    if ( advice == "mozilla" )
        return 3;
    return 0;
}

QString indexToUAName( int idx )
{
    if ( idx == 1 )
       return QString::fromLatin1( "Safari" );
    if ( idx == 2 )
       return QString::fromLatin1( "Internet Explorer" );
    if ( idx == 3 )
       return QString::fromLatin1( "Mozilla" );
   return QString::fromLatin1( "Konqueror" );
}
QString indexToUAString( int idx )
{
    if ( idx == 1 )
       return QString::fromLatin1( "Mozilla/5.0 (Macintosh; U; PPC Mac OS X; appLanguage) AppleWebKit/125.5.5 (KHTML, like Gecko) Safari/125.11" );
    if ( idx == 2 )
       return QString::fromLatin1( "Mozilla/4.0 (compatible; MSIE 6.0; appPlatform; appSysName appMachineType) Konqueror/3.4 (KHTML, like Gecko)" );
    if ( idx == 3 )
       return QString::fromLatin1( "Mozilla/5.0 (appPlatform; U; appSysName appMachineType; appLanguage; rv:1.7.3) Gecko/20040916" );
   return QString::null;
}
}

namespace {
    static const int SPACING = 4;
    static const int MARGIN = 4;

    // helper functions to keep look & feel consistent & reduce code duplication
    QGroupBox *initBox( QGroupBox *box )
    {
        box->setMargin( MARGIN );
        return box;
    }

    QBoxLayout *initLayout( QBoxLayout *l )
    {
        l->setMargin( MARGIN );
        l->setSpacing( SPACING );
        return l;
    }
/*
    void initFontList( QStringList &fontFamilies )
    {
    #ifndef QT_NO_FONTDATABASE
        QFontDatabase fontdb;
        fontFamilies = fontdb.families();
    #else
        if ( !qt_fontmanager )
            QFontManager::initialize();

        QDiskFont *df = qt_fontmanager->diskfonts.first();
        for (; df; df = qt_fontmanager->diskfonts.next() )
            fontFamilies.append( df->name.latin1() );

    #endif

    #if defined(_QT_QPE_)
        // ### ugly
        if ( fontFamilies.count() == 0 )
        {
            QFont defaultFont;
            fontFamilies.append( defaultFont.family() );

            if ( FontManager::hasUnicodeFont() )
            {
                QFont uni1 = FontManager::unicodeFont( FontManager::Proportional );
                QFont uni2 = FontManager::unicodeFont( FontManager::Fixed );

                if ( uni1 != defaultFont )
                    fontFamilies.append( uni1.family() );
                if ( uni2 != uni1 )
                    fontFamilies.append( uni2.family() );
            }
        }
    #endif
    }
*/
    void setCurrentItem( QComboBox *combo, const QString &item )
    {
        for ( int i = 0; i < combo->count(); ++i )
            if ( combo->text( i ).lower() == item.lower() )
            {
                combo->setCurrentItem( i );
                break;
            }
    }
/*
    void setupFontCombo( QComboBox *combo )
    {
        combo->clear();

        QStringList items;
        initFontList( items );

#ifdef _WS_QWS_
        {
            // ### HACK to get rid of the ugly babelfish thingy
            QStringList::Iterator it = items.begin();
            QStringList::Iterator end = items.end();
            for (; it != end; ++it )
                if ( (*it).lower() == QString::fromLatin1( "babelfish" ) )
                {
                    items.remove( it );
                    break;
                }
        }
#endif
        combo->insertStringList( items );
    }
*/
    void populateCookiePolicy( QComboBox* cookiePolicy )
    {
        cookiePolicy->insertItem( i18n( "Ask" ) );
        cookiePolicy->insertItem( i18n( "Reject" ) );
        cookiePolicy->insertItem( i18n( "Accept" ) );
    }
}

static const int HTML_DEFAULT_MED_FONT_SIZE = 10;

// Consists of a set of tabs and shared "Accept/Cancel/Defaults" buttons beneath them.
PreferencesRoad::PreferencesRoad( MainWindowBase* parent, const char* name, WFlags fl )
    : QDialog( parent, name, true, fl ),
      m_mainWindow( parent ),
      homePage( 0 ), currentPage( 0 ),
      cookiePolicy( 0 ), enableJavaScript( 0 ), minimumFontSize( 0 ), mediumFontSize( 0 ),
      //defaultFont( 0 ), 
      defaultModeBar( 0 ), userAgentStringBar( 0 ),
      certBox( 0 ), caList( 0 ), policies( new KConfig("ksslpolicies", false) ),
      _signers( 0 ),
      removeCertBtn( 0 ), verifyCertBtn( 0 ), editCertBtn( 0 ),
      caSSLImport( 0 ), caSSLRemove( 0 ), caSSLRestore( 0 ), caSSLView( 0 ), caAccept( 0 )
{
    setCaption( tr( "Preferences" ) );

    QTabWidget *tabWidget = new QTabWidget( this, "tabWidget" );
    tabWidget->setMargin( 0 );

    QWidget *tab = new QWidget( tabWidget, "generalTab" );
    createGeneralPrefs( tab );
    tabWidget->insertTab( tab, i18n( "General" ) );

    tab = new QWidget( tabWidget, "webFeaturesTab" );
    createWebFeaturesPrefs( tab );
    tabWidget->insertTab( tab, i18n( "Web Features" ) );

    tab = new QWidget( tabWidget, "privacyTab" );
    createPrivacyPrefs( tab );
    tabWidget->insertTab( tab, i18n( "Privacy" ) );

    tab = new QWidget( tabWidget, "certificatesTab" );
    createCertificatePrefs( tab );
    tabWidget->insertTab( tab, i18n( "SSL Certificates" ) );

    tab = new QWidget( tabWidget, "casTab" );
    createCAPrefs( tab );
    tabWidget->insertTab( tab, i18n( "SSL Signers" ) );

    QLayout *tabIndependentControls = createTabIndependentControls( this );

    QBoxLayout *layout = initLayout( new QHBoxLayout( this ) );
    layout->add( tabWidget );
    layout->addLayout( tabIndependentControls );

    init();
    otherCertDelList.setAutoDelete(true);
    caDelList.setAutoDelete(true);
#if defined(_WS_QWS_) && !defined _QT_QPE_
    if ( MainWindowBase::STYLE_WINDOWED != MainWindowBase::s_winStyle )
	showMaximized();
#endif
}

PreferencesRoad::~PreferencesRoad()
{
    delete policies;
    delete _signers;
}

void PreferencesRoad::defaults()
{
//    setCurrentItem( defaultFont, HTML_DEFAULT_VIEW_FONT );
    minimumFontSize->setValue( HTML_DEFAULT_MIN_FONT_SIZE );
    mediumFontSize->setValue( HTML_DEFAULT_MED_FONT_SIZE );

    enableJavaScript->setChecked( true );

    //homePage->setText( DEFAULT_HOMEPAGE );

    cookiePolicy->setCurrentItem( 2 );

    defaultModeBar->setCurrentItem( 0 );

    userAgentStringBar->setCurrentItem( 0 );
}

void PreferencesRoad::init()
{
    KConfig* global = KGlobal::config();

    global->setGroup( "HTML Settings" );
    KHTMLSettings *settings = KHTMLFactory::defaultHTMLSettings();
    minimumFontSize->setValue( global->readNumEntry( "MinimumFontSize", settings->minFontSize() ) );
    mediumFontSize->setValue( global->readNumEntry( "MediumFontSize", settings->mediumFontSize() ) );
//    setCurrentItem( defaultFont, global->readEntry( "StandardFont", settings->stdFontName() ) );

    global->setGroup( "Java/JavaScript Settings" );
    enableJavaScript->setChecked( global->readBoolEntry( "EnableJavaScript", KHTMLFactory::defaultHTMLSettings()->isJavaScriptEnabled() ) );

    global->setGroup( "General" );
    homePage->setText( global->readEntry( "HomeURL", BrowserView::homeURL().prettyURL() ) );
    userAgentStringBar->setCurrentItem( prAnon::UAToIndex( global->readEntry( "UserAgentName" ) ) );

    KConfig cookiecfg ("kcookiejarrc");
    cookiecfg.setGroup ("Cookie Policy");
    cookiePolicy->setCurrentItem(
        prAnon::cookieAdviceToIndex(
            cookiecfg.readEntry( "CookieGlobalAdvice", 0 ) ) );

    // settings that are too ROAD-specific to put into global config
    KConfig roadcfg ("roadrc");
    roadcfg.setGroup ("Default Bar");
    roadcfg.writeEntry( "Index",  defaultModeBar->currentItem() );
    emit( defaultModeSet( defaultModeBar->currentItem() ) );
}

void PreferencesRoad::save()
{
    KConfig* global = KGlobal::config();

    global->setGroup( "HTML Settings" );
//    global->writeEntry( "StandardFont", defaultFont->currentText() );
    global->writeEntry( "MinimumFontSize", minimumFontSize->value() );
    global->writeEntry( "MediumFontSize", mediumFontSize->value() );

    global->setGroup( "Java/JavaScript Settings" );
    global->writeEntry( "EnableJavaScript", enableJavaScript->isChecked() );

    global->setGroup( "General" );
    global->writeEntry( "HomeURL", homePage->text() );
    global->writeEntry( "UserAgentName", prAnon::indexToUAName( userAgentStringBar->currentItem() ) );
    global->writeEntry( "UserAgentString", prAnon::indexToUAString( userAgentStringBar->currentItem() ) );

    global->sync();

    KConfig cookiecfg ("kcookiejarrc");
    cookiecfg.setGroup ("Cookie Policy");
    cookiecfg.writeEntry( "CookieGlobalAdvice", prAnon::indexToCookieAdvice( cookiePolicy->currentItem() ) );
    cookiecfg.sync();

    // settings that are too ROAD-specific to put into global config
    KConfig roadcfg ("roadrc");
    roadcfg.setGroup ("Default Bar");
    roadcfg.writeEntry( "Index",  defaultModeBar->currentItem() );
    roadcfg.sync();
    emit( defaultModeSet( defaultModeBar->currentItem() ) );

    // SSL Policies code
    KSSLCertificateCache _cc;
    for (OtherCertItem *x = otherCertDelList.first(); x != 0; x = otherCertDelList.next())
    {
        KSSLX509Map cert(x->configName());
        QString thisCN = cert.getValue("CN");
        if (!_cc.removeByCN(thisCN))
            kdDebug() <<  "Failed to remove certificate " <<  x->configName().latin1() <<"!" << endl;
    }
    otherCertDelList.clear();

    // Go through the non-deleted ones and save them
    for (OtherCertItem *x =static_cast<OtherCertItem *>(certBox->firstChild());
         x;
         x = static_cast<OtherCertItem *>(x->nextSibling()))
    {
        KSSLX509Map cert(x->configName());
        QString thisCN = cert.getValue("CN");
        QDateTime expires = x->getExpires();
        if (!_cc.modifyByCN(thisCN, (KSSLCertificateCache::KSSLCertificatePolicy)x->getPolicy(), x->isPermanent(), expires))
            kdDebug() << "Failed to modify certificate " << x->configName().latin1() <<"!" << endl;
    }

    bool doGen = false;
    for (CAItem *x = static_cast<CAItem *>(caList->firstChild()); x;
                x = static_cast<CAItem *>(x->nextSibling())) {
        if (!x->modified && !x->isNew) continue;
        if (x->isNew) {
            x->isNew = false;
            _signers->addCA(x->getCert(),
                            x->getSite(),
                            x->getEmail(),
                            x->getCode());
        } else {
            _signers->setUse(x->configName(),
                            x->getSite(),
                            x->getEmail(),
                            x->getCode());
        }
        x->modified = false;
        doGen = true;
    }
    for (CAItem *x = caDelList.first(); x != 0; x = caDelList.next()) {
        _signers->remove(x->configName());
        doGen = true;
    }
    caDelList.clear();

    if (doGen) genCAList();
    // the _cc.removeByCN and _cc.modifyByCN calls above have saved a new
    // version of ksslpolicies.  When this->policies is destroyed, it
    // automatically overwrites that new version.  We update this->policies
    // from the saved file, so the overwriting will have no effect.
    // Note that this is (a) necessary and (b) works *only* because the
    // behaviour of KConfig differs between konqueror and konqE!  If KConfig
    // is ever changed to conform to the konqueror standard, this->policies 
    // should become a KSimpleConfig and "policies->reparseConfiguration()"
    // should be replaced with "policies->sync()".
    policies->reparseConfiguration();
}

void PreferencesRoad::makeCurrentSiteHomePage()
{
    View *view = m_mainWindow->currentView();
    if ( !view || !view->part() )
        return;
    homePage->setText( view->part()->url().prettyURL() );
}

QLayout *PreferencesRoad::createPrivacyPrefs( QWidget *parent )
{
    QBoxLayout *outerLayout = new QVBoxLayout( parent );
    QBoxLayout *layout = initLayout( new QHBoxLayout( 0 ) );
    layout->add( createCookiePrefs( parent ) );
    layout->add( createCachePrefs( parent ) );
    layout->add( createHistoryPrefs( parent ) );
    outerLayout->addLayout( layout );
    outerLayout->addStretch( 1 );
    return layout;
}

QLayout *PreferencesRoad::createGeneralPrefs( QWidget *parent )
{
    QHBox *box = new QHBox( parent );
    box->setSpacing( SPACING );
    box->setMargin( 0 );
    createModePrefs( box );
    createAppearancePrefs( box );

    QBoxLayout *layout = initLayout( new QVBoxLayout( parent ) );
    layout->add( createHomePagePrefs( parent ) );
    layout->add( box );
    layout->addStretch( 1 );
    return layout;
}

QLayout *PreferencesRoad::createWebFeaturesPrefs( QWidget *parent )
{
    QWidget *box = initBox( new QHGroupBox( tr("JavaScript"), parent ) );
    enableJavaScript = new QCheckBox( tr( "Enable" ), box, "enableJavaScript" );

    QBoxLayout *layout = new QVBoxLayout( parent );
    layout->setSpacing( SPACING );
    layout->setMargin( MARGIN );
    layout->add( box );
    layout->add( createUserAgentStringPrefs( parent ) );
    return layout;
}

// Allows a home page URL to be entered into a text field, or 
//  taken from the currently open page.
QWidget *PreferencesRoad::createHomePagePrefs( QWidget *parent )
{
    QWidget *box = initBox( new QHGroupBox( tr( "Home Page" ), parent ) );

    homePage = new QLineEdit( box, "homePage" );
    homePage->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum ) );

    currentPage = new QPushButton( i18n( "  Current  " ), box, "currentPage" );
    currentPage->setEnabled( m_mainWindow->currentView() &&
			     m_mainWindow->currentView()->isPlainBrowser() );
    connect( currentPage, SIGNAL( clicked() ), this, SLOT( makeCurrentSiteHomePage() ) );

    return box;
}

// Set the default mode (URL entry, search, or tabbed browsing).
// The corresponding toolbar is always shown; the others are only shown 
// temporarily, as needed.
// The mode choices in the combo box are created so that their indices match
//  those of the corresponding modes in the main window, so that e.g. selecting
//  the 2cnd combo box option should activate mode #2.
QWidget *PreferencesRoad::createModePrefs( QWidget *parent )
{
    QGroupBox *box = initBox( new QHGroupBox( "Toolbar Configuration", parent ) );

    new QLabel( tr( "Always show:" ), box );
    defaultModeBar = new QComboBox( false, box, "defaultModeBar" );
    //defaultModeBar->listBox()->reparent(this, WStyle_StaysOnTop, QPoint());
    if ( !m_mainWindow->inherits( "MainWindowRoad" ) )
    {
        defaultModeBar->hide();
        return 0;
    }
    MainWindowRoad* win = static_cast<MainWindowRoad*>(m_mainWindow);
    QPtrListIterator<RoadMode> modeIt = win->getModeIterator();
    for ( modeIt.toFirst(); modeIt.current(); ++modeIt)
    {
        RoadMode *mode = modeIt.current();
        defaultModeBar->insertItem( mode->pixmap(), mode->description() );
    }
    defaultModeBar->setCurrentItem( win->getDefaultMode() );
    connect( this, SIGNAL( defaultModeSet( int ) ),
             m_mainWindow, SLOT( setDefaultMode( int ) ) );

    box->addSpace( 0 );
    return box;


}

QWidget *PreferencesRoad::createUserAgentStringPrefs( QWidget *parent )
{
    QWidget *box = initBox( new QHGroupBox( tr( "Identify Browser As" ), parent ) );

    userAgentStringBar = new QComboBox( false, box, "uasBar" );
    userAgentStringBar->insertItem( tr( "Konqueror" ) );
    userAgentStringBar->insertItem( tr( "Safari" ) );
    userAgentStringBar->insertItem( tr( "IE" ) );
    userAgentStringBar->insertItem( tr( "Mozilla" ) );

    return box;
}

QWidget *PreferencesRoad::createCookiePrefs( QWidget *parent )
{
    QWidget *box = initBox( new QVGroupBox(  tr("Cookies" ), parent ) );

    QButton *clearBtn = new QPushButton( tr( "  Clear  " ), box, "delCookiesBtn" );
    clearBtn->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
    connect( clearBtn, SIGNAL( clicked() ), this, SLOT( clearCookies() ) );

    cookiePolicy = new QComboBox( false, box, "cookiePolicy" );
    populateCookiePolicy( cookiePolicy );

    return box;
}

QWidget *PreferencesRoad::createCachePrefs( QWidget *parent )
{
    QWidget *box = initBox( new QVGroupBox( tr( "Cache" ), parent ) );
    QButton *clearBtn = new QPushButton( tr( "  Clear  " ), box, "delCacheBtn" );
    clearBtn->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
    connect( clearBtn, SIGNAL( clicked() ), this, SLOT( clearCache() ) );
    return box;
}

QWidget *PreferencesRoad::createHistoryPrefs( QWidget *parent )
{
    QWidget *box = initBox( new QVGroupBox( tr( "URL History" ), parent ) );
    QButton *clearBtn = new QPushButton( tr( "  Clear  " ), box, "delHistoryBtn" );
    clearBtn->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
    connect( clearBtn, SIGNAL( clicked() ), this->parent(), SIGNAL( clearURLHistoryClicked() ) );
    return box;
}

QLayout *PreferencesRoad::createAppearancePrefs( QWidget *parent )
{
    QWidget *box = initBox( new QVGroupBox( tr( "Font settings" ), parent ) );

    QWidget *subBox = new QHBox( box );
    QLabel *label = new QLabel( 0, tr( "Minimum Font Size:" ), subBox );
    minimumFontSize = new QSpinBox( subBox, "minimumFontSize" );
    label->setBuddy( minimumFontSize );

    subBox = new QHBox( box );
    label = new QLabel( 0, tr( "Medium Font Size:" ), subBox );
    mediumFontSize = new QSpinBox( subBox, "mediumFontSize" );
    label->setBuddy( mediumFontSize );

/*    subBox = new QHBox( box );
    label = new QLabel( 0, tr( "Default Font:" ), subBox );
    defaultFont = new QComboBox( false, subBox, "defaultFont" );
    setupFontCombo( defaultFont );
    label->setBuddy( defaultFont );*/

    QBoxLayout *layout = initLayout( new QVBoxLayout( parent ) );
    layout->add( box );
    layout->addStretch( 1 );
    return layout;
}

QLayout *PreferencesRoad::createCertificatePrefs( QWidget *parent )
{
    QBoxLayout *layout = initLayout( new QHBoxLayout( parent ) );

    layout->add( createCertificateBox( parent, "certBox" ) );
    QWidget *btnParent = new QWidget( parent );
    QBoxLayout *box = new QVBoxLayout( btnParent );
    box->setSpacing( SPACING );
    box->setMargin( 0 );
    removeCertBtn = new QPushButton( tr( "&Remove" ), btnParent, "removeCertBtn" );
    box->add( removeCertBtn );
    verifyCertBtn = new QPushButton( tr( "&Verify" ), btnParent, "verifyCertBtn" );
    box->add( verifyCertBtn );
    editCertBtn = new QPushButton( tr( "&Edit..." ), btnParent, "editCertBtn" );
    box->add( editCertBtn );
    box->addStretch( 1 );
    layout->add(btnParent);

    connect(removeCertBtn, SIGNAL(clicked()), SLOT(removeCert()));
    connect(verifyCertBtn, SIGNAL(clicked()), SLOT(verifyCert()));
    connect(editCertBtn, SIGNAL(clicked()), SLOT(editCert()));
    certSelectionChanged();

    return layout;
}

void PreferencesRoad::certSelectionChanged()
{
    bool b = certBox->selectedItem();
    removeCertBtn->setEnabled(b);
    verifyCertBtn->setEnabled(b);
    editCertBtn->setEnabled(b);
}

void PreferencesRoad::caSelectionChanged()
{
    CAItem *x = static_cast<CAItem *>(caList->selectedItem());
    KSSLCertificate *cert = (x) ? KSSLCertificate::fromString(x->getCert().local8Bit()) : 0;
    bool somethingIsSelected = (0 != x);
    caSSLRemove->setEnabled(somethingIsSelected);
    caSSLView->setEnabled(somethingIsSelected);
    caAccept->setEnabled( (cert) ? cert->x509V3Extensions().certTypeSSLCA() : false );
    caAccept->setChecked( ( x ) ? x->getSite() : false );

}

QLayout *PreferencesRoad::createCAPrefs( QWidget *parent )
{
    QBoxLayout *layout = initLayout( new QHBoxLayout( parent ) );
    layout->add( createCAList( parent, "caList" ) );
    QWidget *btnParent = new QWidget( parent );
    caSSLImport =  new QPushButton( tr( "Import..." ), btnParent, "caSSLImport" );
    caSSLRemove =  new QPushButton( tr( "Remove" ),    btnParent, "eraseCABtn" );
    caSSLRestore = new QPushButton( tr( "Restore" ),   btnParent, "caSSLRestore" );
    caSSLView =    new QPushButton( tr( "View..." ),   btnParent, "caSSLView" );
    caAccept =     new QCheckBox( tr( "Accept" ), btnParent, "acceptCA" );

    QBoxLayout *box = new QVBoxLayout( btnParent );
    box->setSpacing( SPACING );
    box->setMargin( 0 );
    box->add( caSSLImport );
    box->add( caSSLRemove );
    box->add( caSSLRestore );
    box->add( caSSLView );
    box->addStretch( 1 );
    box->add( caAccept );
    layout->add( btnParent );

    connect(caSSLImport,  SIGNAL(clicked()),     SLOT(importCA()));
    connect(caSSLRemove,  SIGNAL(clicked()),     SLOT(removeCA()));
    connect(caSSLRestore, SIGNAL(clicked()),     SLOT(restoreCA()));
    connect(caSSLView,    SIGNAL(clicked()),     SLOT(viewCA()));
    connect(caAccept,     SIGNAL(toggled(bool)), SLOT(caAccepted(bool)));
    caSelectionChanged();

    return layout;
}

QLayout *PreferencesRoad::createTabIndependentControls( QWidget *parent )
{
    QPushButton *defaultsBtn = new QPushButton( tr( " Defaults " ), parent, "defaultsBtn" );
    connect( defaultsBtn, SIGNAL( clicked() ), parent, SLOT( defaults() ) );

    QLayout *layout = new QVBoxLayout;
    layout->setSpacing( SPACING );
    layout->setMargin( 0 );
    layout->addItem( new QSpacerItem( 20, 20 ) );
    layout->add( defaultsBtn );
    layout->addItem( new QSpacerItem( 20, 20, QSizePolicy::Maximum, QSizePolicy::Expanding ) );

#ifndef _QT_QPE_
    QPushButton *okBtn = new QPushButton( tr( "&OK" ), parent, "okBtn" );
    okBtn->setDefault(true);
    QPushButton *cancelBtn = new QPushButton( tr( "&Cancel" ), parent, "cancelBtn" );
    connect( okBtn, SIGNAL( clicked() ), parent, SLOT( accept() ) );
    connect( cancelBtn, SIGNAL( clicked() ), parent, SLOT( reject() ) );
    layout->add( okBtn );
    layout->add( cancelBtn );
    layout->addItem( new QSpacerItem( 20, 1 ) );
#endif

    return layout;
}

QListView *PreferencesRoad::createCertificateBox( QWidget *parent, const char *name )
{
    certBox = new QListView( parent, name );
    certBox->setAllColumnsShowFocus(true);
    certBox->addColumn(i18n("Organization"));
    certBox->addColumn(i18n("Common Name"));

    QTimer::singleShot( 0, this, SLOT(populateCertBox()) );

    return certBox;
}

void PreferencesRoad::populateCertBox()
{
    certBox->setEnabled(false);
    const QStringList &groups = policies->groupList();
    for (QStringList::ConstIterator i = groups.begin(); i != groups.end(); ++i)
    {
	qApp->processEvents();
        if ((*i).isEmpty() || *i == "<default>" || *i == "General") continue;
        policies->setGroup(*i);
        KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
        if (cert)
        {
            new OtherCertItem(certBox, cert->getSubject(), *i,
                            policies->readBoolEntry("Permanent", true),
                            policies->readNumEntry("Policy", 3),
                            policies->readDateTimeEntry("Expires") );
            delete cert;
        }
    }
    connect(certBox, SIGNAL(selectionChanged()), this, SLOT(certSelectionChanged()));
    certBox->setEnabled(true);
}

void PreferencesRoad::clearCookies() const
{
    kapp->dcopClient()->send( "kcookiejar", "kcookiejar", "deleteAllCookies()", QByteArray() );
}

void PreferencesRoad::clearCache() const
{
    KApplication::startServiceByDesktopPath("http_cache_deleter.desktop");
}

void PreferencesRoad::removeCert()
{
    QListViewItem *act = certBox->selectedItem();
    OtherCertItem *x = static_cast<OtherCertItem *>(act);
    if (x) {
        QListViewItem *next = act->itemBelow();
        if (!next) next = act->itemAbove();
        certBox->takeItem(x);
        otherCertDelList.append(x);
        if (next) certBox->setSelected(next, true);
    }
}

void PreferencesRoad::removeCA()
{
    CAItem *x = static_cast<CAItem *>(caList->selectedItem());
    if (x)
    {
        QListViewItem *act = caList->selectedItem();
        QListViewItem *next = act->itemBelow();
        if (!next) next = act->itemAbove();
        caList->takeItem(x);
        caDelList.append(x);
        if (next) caList->setSelected(next, true);
    }
}

void PreferencesRoad::verifyCert() {
    OtherCertItem *x = static_cast<OtherCertItem *>(certBox->selectedItem());
    if (!x) return;

    policies->setGroup(x->getMD5());
    KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());

    if (!cert) {
        KMessageBox::error(this, i18n("Error obtaining the certificate."), i18n("SSL"));
        return;
    }

    cert->chain().setCertChain(policies->readListEntry("Chain"));

    KSSLCertificate::KSSLValidation v = cert->revalidate(KSSLCertificate::SSLServer);

    if (v == KSSLCertificate::Ok) {
        QMessageBox::information(this, i18n("This certificate passed the verification tests successfully."), i18n("SSL"));
    } else {
        QString msgText = i18n("This certificate has failed the tests and should be considered invalid.\nDetails: ");
        KMessageBox::error(this, msgText + KSSLCertificate::verifyText(v), i18n("SSL"));
    }

    delete cert;
}
void PreferencesRoad::editCert()
{
    OtherCertItem *x = static_cast<OtherCertItem *>(certBox->selectedItem());
    if (!x)
        return;
    policies->setGroup(x->getMD5());
    KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
    if (!cert)
        return;
    cert->chain().setCertChain(policies->readListEntry("Chain"));
    EditCertDialog dlg( x, cert, this );
#ifdef _QT_QPE_
    QPEApplication::execDialog(&dlg);
#else
    dlg.exec();
#endif
}

QListView *PreferencesRoad::createCAList( QWidget *parent, const char *name )
{
    caList = new QListView( parent, name );
    caList->setAllColumnsShowFocus(true);
    caList->addColumn(i18n("Organization"));
    caList->addColumn(i18n("Organizational Unit"));
    caList->addColumn(i18n("Common Name"));

    QTimer::singleShot( 0, this, SLOT(populateCAList()) );

    return caList;
}

void PreferencesRoad::populateCAList()
{
    caList->setEnabled(false);
    if (!_signers)
        _signers = new KSSLSigners;
    const QStringList &groups = _signers->list();
    KConfig sigcfg("ksslcalist", true, false);
    for (QStringList::ConstIterator i = groups.begin(); i != groups.end(); ++i) 
    {
	qApp->processEvents();
        if ((*i).isEmpty() || *i == "<default>") continue;
        if (!sigcfg.hasGroup(*i)) continue;
        sigcfg.setGroup(*i);
        if (!sigcfg.hasKey("x509")) continue;
        new CAItem(caList,
            (*i),
            sigcfg.readEntry("x509"),
            sigcfg.readBoolEntry("site", false),
            sigcfg.readBoolEntry("email", false),
            sigcfg.readBoolEntry("code", false));
    }
    connect(caList, SIGNAL(selectionChanged()), this, SLOT(caSelectionChanged()));
    caList->setEnabled(true);
}

void PreferencesRoad::genCAList()
{
   _signers->regenerate();
}

void PreferencesRoad::viewCA()
{
    CAItem *x = static_cast<CAItem *>(caList->selectedItem());
    if (!x)
        return;
    ViewCADialog dlg( x, this );
#ifdef _QT_QPE_
    QPEApplication::execDialog(&dlg);
#else
    dlg.exec();
#endif
}

void PreferencesRoad::restoreCA()
{
    int rc = KMessageBox::warningContinueCancel(this, i18n("This will revert your certificate signers database to the KDE default.\nThis operation cannot be undone.\nAre you sure you wish to continue?"), i18n("SSL"),i18n("Revert"));
        if (rc == KMessageBox::Cancel) {
            return;
        }

    // For now, we just rm the existing file and rebuild
    QString path = KGlobal::dirs()->saveLocation("config");
    path += "/ksslcalist";
    QFile::remove(path);

    // Remove all our old work and rebuild the GUI/List
    caDelList.clear();
    caList->clear();
    populateCAList();
    genCAList();
}

void PreferencesRoad::importCA()
{
    QString certFile = KFileDialog::getOpenFileName(QString::null, "application/x-x509-ca-cert");

    if (certFile.isEmpty())
        return;

#ifdef HAVE_SSL
#define sk_free KOSSL::self()->sk_free
#define sk_num KOSSL::self()->sk_num
#define sk_value KOSSL::self()->sk_value

	// First try to load using the OpenSSL method
	X509_STORE *certStore = KOSSL::self()->X509_STORE_new();
	X509_LOOKUP *certLookup = KOSSL::self()->X509_STORE_add_lookup(certStore, KOSSL::self()->X509_LOOKUP_file());

	if (certLookup &&
	    KOSSL::self()->X509_LOOKUP_load_file(certLookup,
		                                 certFile.local8Bit(),
						 X509_FILETYPE_PEM)) {
		for (int i = 0; i < sk_X509_OBJECT_num(certStore->objs); i++) {
			X509_OBJECT* x5o = sk_X509_OBJECT_value(certStore->objs, i);
			if (!x5o) continue;

			if (x5o->type != X509_LU_X509) continue;

			X509 *x5 = x5o->data.x509;
			if (!x5) continue;

			// Easier to use in this form
			KSSLCertificate *x = KSSLCertificate::fromX509(x5);

			// Only import CA's
			if (!x || !x->x509V3Extensions().certTypeCA()) {
				if (x) {
					QString emsg = x->getSubject() + ":\n" +
					i18n("This is not a signer certificate.");
				 	KMessageBox::error(this,
							   emsg,
						   	   i18n("SSL"));
					delete x;
				}
				continue;
			}

			QString name = x->getSubject();

			// search for dups
			for (CAItem *m = static_cast<CAItem *>(caList->firstChild());
                                                                   m;
        	                 m = static_cast<CAItem *>(m->nextSibling())) {
			         if (m->configName() == name) {
				    KSSLCertificate *y = KSSLCertificate::fromString(m->getCert().local8Bit());
				    if (!y) continue;
				    if (*x == *y) {
					QString emsg = name + ":\n" +
						i18n("You already have this signer certificate installed.");
				 	KMessageBox::error(this,
							   emsg,
						   	   i18n("SSL"));
					 delete x;
					 delete y;
					 x = NULL;
					 break;
			            }
				    delete y;
				 }
			}

			if (!x) continue;

			// Ok, add it to the list
			(new CAItem(caList,
				    name,
				    x->toString(),
				    true, true, true)
			 )->isNew = true;
			delete x;
		}

		  // Can the PEM code be wiped out now?
	} else {   // try to load it manually as a single X.509 DER encoded
		// ASSUMPTION: we only read one certificate in this code
		QFile qf(certFile);
		QString name;
		QString certtext;
		KSSLCertificate *x;
		qf.open(IO_ReadOnly);
		qf.readLine(certtext, qf.size());

		if (certStore) { KOSSL::self()->X509_STORE_free(certStore);
				certStore = NULL; }

		if (certtext.contains("-----BEGIN CERTIFICATE-----")) {
			qf.reset();
			certtext = QString::null;
			while (!qf.atEnd()) {
				QString xx;
				qf.readLine(xx, qf.size());
				certtext += xx;
			}
			certtext = certtext.replace(QRegExp("-----BEGIN CERTIFICATE-----"), QString::null);
			certtext = certtext.replace(QRegExp("-----END CERTIFICATE-----"), QString::null);
			certtext = certtext.stripWhiteSpace();
			certtext = certtext.replace(QRegExp("\n"), QString::null);
		} else {
			// Must [could?] be DER
			qf.close();
			qf.open(IO_ReadOnly);
			char *cr;
			cr = new char[qf.size()+1];
			qf.readBlock(cr, qf.size());
			QByteArray qba;
			qba.duplicate(cr, qf.size());
			certtext = KCodecs::base64Encode(qba);
			delete cr;
		}

		qf.close();

		x = KSSLCertificate::fromString(certtext.latin1());

		if (!x) {
			KMessageBox::sorry(this,
				i18n("The certificate file could not be loaded."),
				i18n("SSL"));
			return;
		}

		if (!x->x509V3Extensions().certTypeCA()) {
			KMessageBox::sorry(this,
					i18n("This is not a signer certificate."),
					i18n("SSL"));
			return;
		}

		name = x->getSubject();

		for (CAItem *i = static_cast<CAItem *>(caList->firstChild());
                                                                   i;
                         i = static_cast<CAItem *>(i->nextSibling())) {
		         if (i->configName() == name) {
			    KSSLCertificate *y = KSSLCertificate::fromString(i->getCert().local8Bit());
			    if (!y) continue;
			    if (*x == *y) {
				 KMessageBox::error(this,
				    i18n("You already have this signer certificate installed."),
				    i18n("SSL"));
				 delete x;
				 delete y;
				 return;
			    }
			    delete y;
			 }
		}

		(new CAItem(caList, name, x->toString(), true, true, true))->isNew = true;

		delete x;
	}


	if (certStore) KOSSL::self()->X509_STORE_free(certStore);
#undef sk_free
#undef sk_num
#undef sk_value
#endif
}

void PreferencesRoad::caAccepted(bool on) {
    CAItem *x = static_cast<CAItem *>(caList->selectedItem());
    if (x && x->getSite() != on)
    {
        x->setSite(on);
        x->modified = true;
    }
}

///////////////////////////////////////////////////////////////////

CompactLayoutBuilder::CompactLayoutBuilder( QWidget *parent ) :
    m_outerLayout( new QHBoxLayout( parent ) ),
    m_layout( new QVBoxLayout( 0 ) )
{
    m_layout->setSpacing( SPACING );
    m_layout->setMargin( MARGIN );
}

CompactLayoutBuilder::~CompactLayoutBuilder()
{
    m_layout->addStretch( 1 );
    m_outerLayout->addLayout( m_layout );
    m_outerLayout->addStretch( 1 );
}

QBoxLayout *CompactLayoutBuilder::getLayout()
{
    return m_layout;
}

//////////////////////////////////////////////////////////////////////

OtherCertItem::OtherCertItem( QListView *view, const QString& sub, const QString& md5, bool perm, int policy, QDateTime exp )
    : QListViewItem( view, QString::null ), _sub(sub), _md5(md5), _exp(exp), _perm(perm), _policy(policy)

{
    KSSLX509Map cert(sub);
    setText(0, cert.getValue("O"));
    setText(1, cert.getValue("CN").replace( QRegExp("\n"), ", "));

    if (_exp.date().year() > 3000 || _exp.date().year() < 1900)
       _exp.setDate(QDate(3000,1,1));
}

QString OtherCertItem::configName() const
{
    return _sub;
}

/////////////////////////////////////////////////////////////////////////
CAItem::CAItem( QListView *view, QString name, QString cert, bool site, bool email, bool code )
    : QListViewItem( view, QString::null )

{
    KSSLX509Map mcert(name);
    QString tmp;
    setText(0, mcert.getValue("O"));
    tmp = mcert.getValue("OU");
    tmp.replace( QRegExp( "\n" ), ", ");
    setText(1, tmp);
    _commonName = mcert.getValue("CN");
    _commonName.replace( QRegExp( "\n" ), ", ");
    setText(2, _commonName);
    _name = name;
    _cert = cert;
    _site = site;
    _email = email;
    _code = code;
    isNew = false;
    modified = false;
}

QString CAItem::configName() const
{
    return _name;
}

///////////////////////////////////////////////////////////////////////////////////
EditCertDialog::EditCertDialog( OtherCertItem *x, KSSLCertificate *cert, QWidget *parent, const char * name ) :
    QDialog( parent, name, true ),
    certItem( x ), p_cert(cert)
{
    KSSLX509Map certmap(cert->getSubject());
    setCaption( certmap.getValue("O") + tr("  SSL Certificate") );

    QBoxLayout *mainlayout = new QHBoxLayout( this );
    QScrollView *scrollView = new QScrollView( this );
    QWidget *w = new QWidget( scrollView->viewport() );
    scrollView->addChild( w );
    scrollView->viewport()->setBackgroundMode(w->backgroundMode());
    mainlayout->addWidget( scrollView );

    QVBoxLayout *vLayout = new QVBoxLayout( w, KDialog::marginHint(), 0 );
    QGridLayout *layout = new QGridLayout(5, 2, KDialog::spacingHint());
    vLayout->addItem(layout);

    QHBoxLayout *hlayout = new QHBoxLayout( 0 );
    layout->addMultiCellLayout(hlayout, 0, 0, 0, 1);


    hlayout->addWidget(new QLabel(i18n("Chain:"), w));
    hlayout->addSpacing( 6 );
    _chain = new QComboBox(w);
    _chain->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed) );
    hlayout->addWidget( _chain);
    hlayout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum));
    connect(_chain, SIGNAL(activated(int)), this, SLOT(slotChain(int)));

    _chain->clear();

    if (cert->chain().isValid() && cert->chain().depth() > 1) {
        _chain->setEnabled(true);
        _chain->insertItem(i18n("0 - Site Certificate"));
        int cnt = 0;
        QPtrList<KSSLCertificate> cl = cert->chain().getChain();
        for (KSSLCertificate *c = cl.first(); c != 0; c = cl.next()) {
            KSSLX509Map map(c->getSubject());
            QString id;
            id = map.getValue("CN");
            if (id.length() == 0)
                id = map.getValue("O");
            if (id.length() == 0)
                id = map.getValue("OU");
            _chain->insertItem(QString::number(++cnt)+" - "+id);
        }
        _chain->setCurrentItem(0);
    } else _chain->setEnabled(false);

    oSubject = new KSSLCertBox( w );
    oSubject->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred ) );
    oIssuer = new KSSLCertBox( w );
    oIssuer->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred ) );
    oSubject->setValues(certItem->getSub());
    oIssuer->setValues(cert->getIssuer());
/*
    vLayout->addWidget( new QLabel( tr( "Subject" ), w ) );
    vLayout->addWidget(oSubject);
    vLayout->addWidget( new QLabel( tr( "Issuer" ), w ) );
    vLayout->addWidget(oIssuer);
*/
    layout->addWidget( new QLabel( tr( "Subject" ), w ), 2,0 );
    layout->addWidget(oSubject, 3,0);
    layout->addWidget( new QLabel( tr( "Issuer" ), w ), 2,1 );
    layout->addWidget(oIssuer, 3,1);
    layout->addItem(new QSpacerItem(0,4));

    QPalette pal;
    const QDateTime& curDate = QDateTime::currentDateTime();
    QGridLayout *grid = new QGridLayout( 0, 4, 3, 0, KDialog::spacingHint());
    vLayout->addLayout( grid );
    vLayout->addSpacing(4);

    QLabel *fromLabel = new QLabel(i18n("Valid from:"), w);
    QLabel *untilLabel = new QLabel(i18n("Valid until:"), w);
    grid->addWidget(fromLabel, 0, 0);
    grid->addWidget(untilLabel, 1, 0);

    QLabel *validFrom = new QLabel(cert->getNotBefore(), w);
    pal = validFrom->palette();
    pal.setColor(QColorGroup::Foreground,
        (curDate < cert->getQDTNotBefore())
        ? QColor(196,33,21)
        : QColor(42,153,59));
    validFrom->setPalette(pal);
    grid->addWidget(validFrom, 0, 1);

    QLabel *validUntil = new QLabel(cert->getNotAfter(), w);
    pal = validUntil->palette();
    pal.setColor(QColorGroup::Foreground, 
        (curDate > cert->getQDTNotAfter())
        ? QColor(196,33,21)
        : QColor(42,153,59));
    validUntil->setPalette(pal);
    grid->addWidget(validUntil, 1, 1);

    QButtonGroup *cacheGroup = new QVButtonGroup(i18n("Cache"), w);
    QRadioButton *cachePerm = new QRadioButton(i18n("Permanentl&y"), cacheGroup);
    QRadioButton *cacheUntil = new QRadioButton(i18n("&Until"), cacheGroup);
    untilDate = new QLabel(cacheGroup);
    untilDate->setText( KGlobal::locale()->formatDateTime( certItem->getExpires()) );
    untilDate->setEnabled(!certItem->isPermanent());
    grid->addWidget( cacheGroup, 2, 0 );
    connect(cachePerm, SIGNAL(clicked()), SLOT(slotPermanent()));
    connect(cacheUntil, SIGNAL(clicked()), SLOT(slotUntil()));

    policyGroup = new QVButtonGroup(i18n("Policy"), w );
    policyAccept = new QRadioButton(i18n("Accep&t"), policyGroup);
    policyReject = new QRadioButton(i18n("Re&ject"), policyGroup);
    policyPrompt = new QRadioButton(i18n("&Prompt"), policyGroup);
    grid->addWidget( policyGroup, 2, 1 );
    connect(policyGroup, SIGNAL(clicked(int)), SLOT(slotPolicyChanged(int)));
    switch(certItem->getPolicy()) {
    case KSSLCertificateCache::Accept:
        policyGroup->setButton(policyGroup->id(policyAccept));
        break;
    case KSSLCertificateCache::Reject:
        policyGroup->setButton(policyGroup->id(policyReject));
        break;
    case KSSLCertificateCache::Prompt:
        policyGroup->setButton(policyGroup->id(policyPrompt));
        break;
    default:
        policyGroup->setButton(policyGroup->id(policyPrompt));
        slotPolicyChanged(policyGroup->id(policyPrompt));
    }
    cachePerm->setChecked(certItem->isPermanent());
    cacheUntil->setChecked(!certItem->isPermanent());

//    QHBoxLayout *md5Layout = new QHBoxLayout(0);
//    md5Layout->addWidget(
    vLayout->addWidget(new QLabel(i18n("MD5 digest:  ")+ cert->getMD5DigestText(), w));
//    QLabel* md5 = new QLabel( , w);
//    md5Layout->addWidget( );

//    vLayout->addLayout(md5Layout);
}


void EditCertDialog::slotChain(int x) {
    if (x == 0) {
//        displayCert(p_cert);
    oSubject->setValues(p_cert->getSubject());
    oIssuer->setValues(p_cert->getIssuer());

    } else {
        QPtrList<KSSLCertificate> cl = p_cert->chain().getChain();
        cl.setAutoDelete(true);
        for (int i = 0; i < x-1; i++)
            cl.remove((unsigned int)0);
        KSSLCertificate thisCert = *(cl.at(0));
        cl.remove((unsigned int)0);
        thisCert.chain().setChain(cl);
        //displayCert(&thisCert);
        oSubject->setValues(thisCert.getSubject());
        oIssuer->setValues(thisCert.getIssuer());
    }
}


void EditCertDialog::slotUntil()
{
    untilDate->setEnabled(true);
    if (certItem)
        certItem->setPermanent(false);
}

void EditCertDialog::slotPermanent()
{
    untilDate->setEnabled(false);
    if (certItem)
        certItem->setPermanent(true);
}

void EditCertDialog::slotPolicyChanged(int id) {
    if (!certItem) return;

    if (id == policyGroup->id(policyAccept)) {
        certItem->setPolicy(KSSLCertificateCache::Accept);
    } else if (id == policyGroup->id(policyReject)) {
        certItem->setPolicy(KSSLCertificateCache::Reject);
    } else if (id == policyGroup->id(policyPrompt)) {
        certItem->setPolicy(KSSLCertificateCache::Prompt);
    }
}

//////////////////////////////////////////////////////////////////////////////

ViewCADialog::ViewCADialog(CAItem *x, QWidget *parent, const char * name) :
    QDialog( parent, name, true )
{
    if (!x) return;
    setCaption( x->getCommonName() );

    QBoxLayout *layout = new QHBoxLayout( this );
    QScrollView *scrollView = new QScrollView( this );
    QWidget *w = new QWidget( scrollView->viewport() );
    scrollView->addChild( w );
    scrollView->viewport()->setBackgroundMode(w->backgroundMode());
    layout->addWidget( scrollView );

    QVBoxLayout *toplayout = new QVBoxLayout( w, KDialog::marginHint(), 0 );

    toplayout->addWidget( new QLabel( i18n( "Subject" ), w )) ;
    KSSLCertBox *caSubject = new KSSLCertBox( w );
    caSubject->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred ) );
    caSubject->setValues(x->getName());
    toplayout->addWidget(caSubject);

    toplayout->addWidget(new QLabel( i18n( "Issuer" ), w ));
    KSSLCertBox *caIssuer = new KSSLCertBox( w );
    caIssuer->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred ) );
    KSSLCertificate *cert = KSSLCertificate::fromString(x->getCert().local8Bit());
    toplayout->addWidget(caIssuer);

    QString cHash;
    if (!cert) {
       caIssuer->setValues(QString(QString::null));
       cHash = QString::null;
    } else {
       caIssuer->setValues(cert->getIssuer());
       cHash = cert->getMD5DigestText();
       delete cert;
    }
    toplayout->addWidget(new QLabel(i18n("MD5 digest: %1").arg(cHash), w) );
}

#include "preferences_road.moc"
#endif // KONQ_GUI_ROAD
