/*  This file is part of the KDE project
    Copyright (C) 2001 Simon Hausmann <hausmann@kde.org>
    Copyright (C) 2006 basysKom GmbH <info@basyskom.de>
    Copyright (C) 2006 ROAD GmbH <info@road-gmbh.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef __preferencesroadimpl_h__
#define __preferencesroadimpl_h__

#include <qvariant.h>
#include <qdialog.h>
#include <qlistview.h>

class QSpinBox;
class QListView;
class QCheckBox;
class OtherCertItem;
class QButtonGroup;
class QRadioButton;
class KSSLSigners;
class CAItem;
class KSSLCertBox;
class MainWindowBase;

class PreferencesRoad : public QDialog
{
    Q_OBJECT

public:
    PreferencesRoad( MainWindowBase* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~PreferencesRoad();
    void init();
    void save();
signals:
    void defaultModeSet( int modeIdx );
public slots:
    virtual void defaults();
    virtual void makeCurrentSiteHomePage();

private slots:
    void clearCookies() const;
    void clearCache() const;

    void removeCert();
    void verifyCert();
    void editCert();
    void removeCA();
    void viewCA();
    void restoreCA();
    void importCA();

    void certSelectionChanged();
    void caSelectionChanged();
    void populateCertBox();
    void populateCAList();
    void caAccepted(bool);
private:
    // helper methods to keep the constructor legible
    QLayout *createGeneralPrefs( QWidget *parent );
    QLayout *createPrivacyPrefs( QWidget *parent );
    QLayout *createWebFeaturesPrefs( QWidget *parent );
    QWidget *createHomePagePrefs( QWidget *parent );
    QWidget *createModePrefs( QWidget *parent );
    QWidget *createUserAgentStringPrefs( QWidget *parent );
    QLayout *createAppearancePrefs( QWidget *parent );
    QWidget *createCookiePrefs( QWidget *parent );
    QWidget *createCachePrefs( QWidget *parent );
    QWidget *createHistoryPrefs( QWidget *parent );
    QLayout *createCertificatePrefs( QWidget *parent );
    QLayout *createCAPrefs( QWidget *parent );
    QLayout *createTabIndependentControls( QWidget *parent );
    QListView *createCertificateBox( QWidget *parent, const char *name );
    QListView *createCAList( QWidget *parent, const char *name );
    void genCAList();

    MainWindowBase *m_mainWindow;
    // we need to store these because we must reset their values
    // when the user clicks on "restore defaults".
    QLineEdit* homePage;
    QPushButton* currentPage;
    QComboBox* cookiePolicy;
    QCheckBox* enableJavaScript;
    QSpinBox* minimumFontSize;
    QSpinBox* mediumFontSize;
    //QComboBox* defaultFont;
    QComboBox *defaultModeBar;
    QComboBox *userAgentStringBar;
    QListView *certBox;
    QListView *caList;

    // certificiate related (if it gets to be too much, make a separate class for cert panel)
    KConfig *policies;
    KSSLSigners *_signers;
    QPtrList<OtherCertItem> otherCertDelList;
    QPtrList<CAItem> caDelList;
    QPushButton *removeCertBtn;
    QPushButton *verifyCertBtn;
    QPushButton *editCertBtn;
    QPushButton *caSSLImport;
    QPushButton *caSSLRemove;
    QPushButton *caSSLRestore;
    QPushButton *caSSLView;
    QCheckBox *caAccept;
};

// helper class to reduce code duplication:
// creates a standard layout set; widgets to be laid out are added to the
//  layout returned by getLayout (which remains valid after Builder is desroyed)
// Note that this is not a QObject and layoutParent isn't the Builder's parent!
class CompactLayoutBuilder
{
public:
    CompactLayoutBuilder( QWidget *layoutParent );
    ~CompactLayoutBuilder();
    QBoxLayout *getLayout();
private:
    QBoxLayout *m_outerLayout;
    QBoxLayout *m_layout;
};

class OtherCertItem : public QListViewItem
{
public:
    OtherCertItem(QListView *view, const QString& sub, const QString& md5, bool perm, int policy, QDateTime exp );
    ~OtherCertItem() {}

    QString configName() const;
    const QString& getSub() { return _sub; }
    int getPolicy() { return _policy; }
    const QString& getMD5() { return _md5; }
    bool isPermanent() { return _perm; }
    QDateTime getExpires() { return _exp; }
    void setPolicy(int x) { _policy = x; }
    void setPermanent(bool x) { _perm = x; }
    void setExpires(QDateTime x) { _exp = x; }

private:
    QString _sub, _md5;
    QDateTime _exp;
    bool _perm;
    int _policy;
};

class CAItem : public QListViewItem
{
public:
    CAItem(QListView *view, QString name, QString cert, bool site, bool email, bool code );
    ~CAItem() {}

    QString configName() const;
    QString& getName() { return _name; }
    void setName(QString name) { _name = name; }

    inline QString getCommonName() const { return _commonName; }
    inline QString getCert() const { return _cert; }
    inline bool getSite() const { return _site; }
    inline bool getEmail() const { return _email; }
    inline bool getCode() const { return _code; }
    inline void setSite(bool x) { _site = x; }
    inline void setEmail(bool x) { _email = x; }
    inline void setCode(bool x) { _code = x; }

    bool isNew, modified;

private:
    QString _commonName;
    QString _name;
    QString _cert;
    bool _site, _email, _code;
};


class EditCertDialog : public QDialog
{
    Q_OBJECT
public:
    EditCertDialog(OtherCertItem *, KSSLCertificate *, QWidget *parent, const char * name=0);
private slots:
    void slotUntil();
    void slotPermanent();
    void slotPolicyChanged(int id);
    void slotChain(int x);

private:
    OtherCertItem *certItem;
    KSSLCertificate * p_cert;
    QLabel *untilDate;
    QButtonGroup *policyGroup;
    QRadioButton *policyAccept;
    QRadioButton *policyReject;
    QRadioButton *policyPrompt;
    QComboBox* _chain;
    KSSLCertBox *oSubject;
    KSSLCertBox *oIssuer;
};

class ViewCADialog : public QDialog
{
public:
    ViewCADialog(CAItem *, QWidget *parent, const char * name=0);
};

#endif
