/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/
 
#include <config.h>

#ifdef KONQ_GUI_ROAD
#include "roadmode.h"
#include "toggletoolbutton.h"

#include <qmovie.h>

#if (QT_VERSION < 0x030000)
#define ROAD_DOCK_TOP QMainWindow::Top
#define ROAD_DOCK_BOTTOM QMainWindow::Bottom
#else
#define ROAD_DOCK_TOP Qt::DockTop
#define ROAD_DOCK_BOTTOM Qt::DockBottom
#endif

ModeBar::ModeBar( const QIconSet& pinUpIS, const QIconSet& pinDownIS, QMainWindow *win, const char *name )
    : QToolBar( QString::null, win, ROAD_DOCK_TOP, false, name ),
    m_blockAutoHide( false ),
    m_pinBtn( 0 ), 
    m_loadStatus( 0 ),
    m_pinUpIcons( pinUpIS ),
    m_pinDownIcons( pinDownIS )
{
}

// We must ensure that the pin button is the rightmost of the toolbar widgets.
// Unfortunately, QToolBar::boxLayout() is private in Qt2, and there is no way
// to remove a widget once added, so must simply add the pin button last.
void ModeBar::finalize()
{
    m_pinBtn = new QToolButton( this );
    m_pinBtn->setToggleButton( true );
    m_pinBtn->setOnIconSet( m_pinDownIcons );
    m_pinBtn->setOffIconSet( m_pinUpIcons );
    m_pinBtn->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );

    m_loadStatus = new QLabel( this );
    m_loadStatus->setMinimumWidth(20);
    m_loadStatus->setMovie( KGlobal::dirs()->findResource("icon","Throbber.gif") );
    m_loadStatus->setBackgroundMode(PaletteButton);
    m_loadStatus->hide();

    m_loadLabel = new QLabel( this );
    m_loadLabel->setPixmap( QPixmap( KGlobal::dirs()->findResource("icon","Throbber.png") ));
    m_loadLabel->setMinimumWidth(20);
    m_loadLabel->setBackgroundMode(PaletteButton);
    m_loadLabel->hide();
}

void ModeBar::showPinButton( bool b )
{
    if (m_pinBtn)
        (b) ? m_pinBtn->show() : m_pinBtn->hide();
    if (m_loadStatus)
    {
        (b) ? m_loadStatus->hide() : m_loadStatus->hide();
        if (b) m_loadStatus->movie()->pause();
    }
    if (m_loadLabel)
        (b) ? m_loadLabel->hide() : m_loadLabel->show();
}

void ModeBar::setThrobble( bool b )
{
    if (m_loadStatus)
    {
        (b) ? m_loadStatus->show() : m_loadStatus->hide();
      //  (b) ? m_loadStatus->movie()->pause() : m_loadStatus->movie()->restart();
    }
    if (m_loadLabel)
        (b) ? m_loadLabel->hide() : m_loadLabel->show();
}


void ModeBar::autoHide()
{
    if (isVisible() && !m_blockAutoHide && !( m_pinBtn && m_pinBtn->isOn()))
    {
        hide();
        emit autoHidden();
    }
    // we need to check the pinBtn manually, rather than just connect()ing to blockAutoHide(),
    // because being pinned is a more persistent state than blockAutoHide; e.g. if you were
    // the default bar, then you aren't, your autoHide gets turned on, then off, but your
    // pinned state is unaffected.
}

void ModeBar::blockAutoHide( bool b )
{
    m_blockAutoHide = b;
}

/////////////////////////////////////////////////////////////////////////

RoadMode::RoadMode( ModeBar *bar, ToggleToolButton *btn,
                    const char *description, bool dockBottom )
    : m_bar( bar ),
      m_btn( btn ),
      m_desc( i18n( description ) ),
      m_dockPos( (dockBottom) ? ROAD_DOCK_BOTTOM : ROAD_DOCK_TOP )
{
    m_bar->setHorizontalStretchable( true );
    associate( m_btn, m_bar );
    QObject::connect( m_bar, SIGNAL( autoHidden() ), m_btn, SLOT( toggle() ) );
}

void RoadMode::setDefault( bool b, int defaultModeToolBarIdx )
{
    if (m_bar && m_btn)
    {
        m_bar->blockAutoHide( b );
        (b)
            ? m_bar->mainWindow()->moveToolBar( m_bar, ROAD_DOCK_TOP, false, defaultModeToolBarIdx )
            : m_bar->mainWindow()->moveToolBar( m_bar, m_dockPos,     true,  -1 );
        m_bar->showPinButton( !b );

        if (b)
        {
            m_btn->setOn(true);  // this automatically shows the associated mode bar
            m_btn->hide();
        }
        else
            m_btn->show();
    }
}

QPixmap RoadMode::pixmap()
{
    return (m_btn) ? m_btn->iconSet().pixmap() : QPixmap();
}

#include "roadmode.moc"

#undef ROAD_DOCK_TOP
#undef ROAD_DOCK_BOTTOM
#endif // KONQ_GUI_ROAD
