/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef __roadmode_h__
#define __roadmode_h__

// A RoadMode manages the widgets associated with a user input mode in the ROAD
//  browser, e.g. URL entry, search, or tabbed browsing.
// Each mode has:
//  a ModeBar (see below) that implements the mode's functionality
//   (i.e. a text entry field and a search engine selector for the search mode),
//  a toggle button (on a separate toolbar) that shows or hides the mode bar, and
//  a "pin" toggle button (see below).
// Modes also have the notion of a "default mode".  Default and non-default
//  modes differ as follows:
//   the default mode's mode bar is part of the main browser toolbar;
//    each non-default mode bars has its own line underneath.
//   the default mode's mode bar is always shown, so its show/hide and pin
//    buttons are hidden.
//   a non-default mode's mode bar disappears after a (bar-dependent) action
//    has been performed on it (e.g. a search started),
//    unless it has been pinned, in which case it is visible until
//    explicitly hidden using its show/hide button.

class ModeBar : public QToolBar
{
    Q_OBJECT
public:
    ModeBar( const QIconSet& pinUpIconSet, const QIconSet& pinDownIconSet, QMainWindow *, const char *name );
    void finalize(); // call this after all subwidgets have been added
    void blockAutoHide( bool );
    void showPinButton( bool );
    void setThrobble( bool );
public slots:
    void autoHide();
signals:
    void autoHidden();
private:
    bool m_blockAutoHide;
    QToolButton *m_pinBtn;
    // these only have to be members because of finalize()
    QIconSet m_pinUpIcons;
    QIconSet m_pinDownIcons;
    QLabel *m_loadStatus;
    QLabel *m_loadLabel;
};

/////////////////////////////////////////////////////////////////////////
class ToggleToolButton;

class RoadMode
{
public:
     RoadMode( ModeBar *bar, ToggleToolButton *btn, const char *description,
               bool dockBottom = false );
     void setDefault( bool, int defaultModeToolBarIdx );
     QPixmap pixmap();
     ModeBar* modeBar() { return m_bar; }
     const QString& description() { return m_desc; }
private:
     QGuardedPtr<ModeBar> m_bar;
     QGuardedPtr<ToggleToolButton> m_btn;
     QString m_desc;
#if (QT_VERSION < 0x030000)
     QMainWindow::ToolBarDock m_dockPos;
#else
     Qt::Dock m_dockPos;
#endif
};

#endif
