/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef __roadtabbar_h__
#define __roadtabbar_h__

#include <qtabbar.h>

struct RoadTabParams
{
    RoadTabParams( const QIconSet &defaultIS, const QIconSet &loadingIS, 
                   const QFontMetrics& fm, int maxWidth );
    QIconSet defaultIconSet;
    QIconSet loadingIconSet;
    QFontMetrics fontMetrics;
    int maxWidth;
};

/*
  Tabbed browsing widget for the ROAD browser:
    Clicking on a tab shows the corresponding page
    The context menu is the same as that for the page itself.
*/
class RoadTabBar : public QTabBar
{
    Q_OBJECT
public:
    RoadTabBar( const QIconSet &defaultIS, const QIconSet &loadingIS,
                QWidget *parent=0, const char *name=0 );
    virtual bool event ( QEvent * );
public slots:
    virtual void setCurrentTab ( int );
    virtual void setCurrentTab ( QTab * );
    void newView( View * );
    void removeTab( int tabIdx );
    void tabChanged();
signals:
    // emitted when the active tab changes, but, unlike selected(),
    // not when show() is called!
    void selectedNotShow( int );
    // emitted when a tab is closed _using the context menu_
    void closed( int );
    void requestNewView();
private slots:
    void contextClose();
private:
    void initContextMenu();
    void initTabParams();
    void showContextMenu( QMouseEvent *e );
    // return the index of the given tab, or -1 if it isn't found
    int index( QTab * );

    int m_contextMenuTabIdx;
    QPopupMenu m_contextMenu;
    RoadTabParams m_tabParams;
    int m_contextMenuCloseId;
};

///////////////////////////////////////////////////////////////////////

class RoadTab : public QObject, public QTab
{
    Q_OBJECT
public:
    RoadTab( const RoadTabParams &, const QString &text = QString() );
signals:
    void changed();
public slots:
    void setLoading( bool );
    void setCaption( const QString & caption );
private:
    QString formatCaption( const QString& text );

    const RoadTabParams &m_params; // we can store this reference because the original is stored in RoadTabBar
};

#endif
