/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <config.h>

#ifdef KONQ_GUI_ROAD
#include "toggletoolbutton.h"

ToggleToolButton::ToggleToolButton( const QIconSet &iconSet, QToolBar *parent, const char *name ) :
    QToolButton( iconSet, QString::null, QString::null, 0,
                 0, parent, name )
{
    setToggleButton( true );
    // we connect manually rather than using QToolButton's receiver/slot params, because 
    // we want to react to all toggled, not just clicks.
    connect( this, SIGNAL( toggled( bool ) ), this, SLOT( emitNoParamToggled( bool ) ) );
}

void ToggleToolButton::emitNoParamToggled( bool b )
{
    ( b ) ? emit toggledOn() : emit toggledOff();
}
////////////////////////////////////////////////////////////////////////////////////////
OverlayLabelButton::OverlayLabelButton( const QIconSet &iconSet,
                                         QToolBar *parent, const char *name ) :
    ToggleToolButton( iconSet, parent, name )
{
}

void OverlayLabelButton::drawButtonLabel( QPainter * p )
{
    ToggleToolButton::drawButtonLabel( p );
    if ( text().isNull() && !textLabel().isNull() )
    {
        int x, y, w, h;
        style().toolButtonRect(0, 0, width(), height() ).rect( &x, &y, &w, &h );
        p->setFont( font() );
        const int textWidth = p->fontMetrics().width( textLabel() );
        style().drawItem( p, x + w - textWidth, y, textWidth, h / 2, // HACK: this is hard coded for the ROAD browser tab button; if this class is ever used more generally, it needs a system to position the label as desired
                          AlignCenter + ShowPrefix,
                          colorGroup(), isEnabled(),
                          0, textLabel() );
    }
}

////////////////////////////////////////////////////////////////////////////////////////

void associate( ToggleToolButton *ttb, QWidget *w )
{
    QObject::connect( ttb, SIGNAL( toggledOff() ), w, SLOT( hide() ) );
    QObject::connect( ttb, SIGNAL( toggledOn() ),  w, SLOT( show() ) );
    (ttb->isOn()) ? w->show() : w->hide();
}

#include "toggletoolbutton.moc"
#endif // KONQ_GUI_ROAD
