/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef __toggletoolbutton_h__
#define __toggletoolbutton_h__
/*
    When toggled, a ToggleAction emits not only the
    toggled( bool ) signal, but also toggled{On,Off}.

    This is an adapter to enable connection of a toggle button and an object
     whose toggled behaviour depends on two distinct methods (e.g. a QWidget
     that calls show() or hide() based on button state).
*/
class ToggleToolButton : public QToolButton
{
    Q_OBJECT
public:
    ToggleToolButton( const QIconSet &iconSet, QToolBar *parent, const char *name = 0 );
signals:
    void toggledOff();
    void toggledOn();
private slots:
    void emitNoParamToggled( bool );
};

// "associate" links the toggle and the widget so that
//  the widget is hidden iff the toggle is on.  The link
//  is one-way (since QWidget emits no signal when hidden),
//  so users of the widget should ensure that it can only be
//  hidden using the associated toggle.
void associate( ToggleToolButton *ttb, QWidget *w );

/*
    Displays the text specified in setLabelText (if any) on top of the pixmap
    rather than underneath it.
*/
class OverlayLabelButton : public ToggleToolButton
{
public:
    OverlayLabelButton( const QIconSet &iconSet, QToolBar *parent, const char *name = 0 );
    virtual void setUsesTextLabel( bool /*enable*/ ) {};
protected:
    virtual void drawButtonLabel( QPainter * p );
};

#endif
