/*  This file is part of the KDE project
    Copyright (C) 2005-2006 ROAD Gmbh <info@road-gmbh.de>
    Copyright (C) 2005-2006 basysKom Gmbh <info@basyskom.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <config.h>

#ifdef KONQ_GUI_ROAD
#include "websearcher.h"
#include <qlineedit.h>

const QChar WebSearcher::ENGINE_DESC_SEPARATOR = ',';
const uint WebSearcher::NUM_ITEMS_IN_ENGINE_DESC = 3;

WebSearcher::WebSearcher( ModeBar *parent, const char *name ) :
    QObject( parent, name ),
    m_modeBar( parent ),
    m_activeSearchEngineIdx( 0 )
{
    initSearchEngineList();
}

void WebSearcher::activate( QLineEdit *input, int searchEngineIndex )
{
    m_activeSearchEngineIdx = searchEngineIndex; // keep track of this for when the search is performed
    connect( input, SIGNAL( returnPressed() ), this, SLOT( search() ) );
    // unlike find in page, a web search is usually only performed once, so we hide the search bar afterwards.
    connect( input, SIGNAL( returnPressed() ), m_modeBar, SLOT( autoHide() ) );
}

void WebSearcher::search()
{   // this is part of MainWindowRoad rather than View because the choice of
    //  is global, not by view.
    if ( !m_view || m_engines.size() <= m_activeSearchEngineIdx )
        return;
    SearchEngineData *data = m_engines[m_activeSearchEngineIdx];
    if (!data)
        return;
    KURL searchURL( data->baseSearchURL );
    searchURL.setQuery( data->baseQuery + m_view->findText() );
    m_view->openURL( searchURL );
}

void WebSearcher::connectView( View *v )
{
    m_view = v;
}

void WebSearcher::initSearchEngineList()
{
    // the [Search Engines] group consists of a list of engines, each of which
    // maps to a list of the form RelativeFavIconPath,BaseSearchURL,BaseQueryString
    KConfig *config = KGlobal::config();
    const QMap<QString,QString>& engineDescs = config->entryMap("Search Engines");
    m_engines.resize( engineDescs.count() );
    int i = 0;
    const KStandardDirs* stdDirs = KGlobal::dirs();
    for( QMap<QString,QString>::ConstIterator it = engineDescs.begin();
         it != engineDescs.end(); ++it, ++i )
    {
        QStringList desc = QStringList::split( ENGINE_DESC_SEPARATOR, it.data() );
        if ( desc.count() != NUM_ITEMS_IN_ENGINE_DESC )
            continue;
        QStringList::ConstIterator descIt = desc.begin();
        SearchEngineData *data = new SearchEngineData();
        data->pixmap = QPixmap(stdDirs->findResource("icon",*descIt));
        data->baseSearchURL = *(++descIt);
        data->baseQuery = *(++descIt);
        m_engines.insert(i, data);
    }
    m_engines.setAutoDelete( true );
}

uint WebSearcher::getNumEngines() const
{
    return m_engines.size();
}

SearchEngineData *WebSearcher::getEngineData( uint i )
{
    return ( m_engines.size() > i ) ? m_engines[i] : 0;
}

#include "websearcher.moc"
#endif // KONQ_GUI_ROAD
